`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:    15:47:59 01/06/06
// Design Name:    
// Module Name:    CONV_GEN
// Project Name:   
// Target Device:  
// Tool versions:  
// Description:
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////
module CONV_GEN(	DDC_CLK,	
						FPGA_RESET,
						CONV_LOW_REG,
						CONV_HIGH_REG,
						CONV_WAIT_LOW_REG,
						CONV_WAIT_HIGH_REG,
						CONV_OUT,
						CONV_SEL,
						NON_CONT,
						RESET_CONV,
						PAUSE_FOR_CONV,
						CLKDELAY_AROUND_CONV
						);

input DDC_CLK;
input FPGA_RESET;
input [23:0] CONV_LOW_REG;
input [23:0] CONV_HIGH_REG;
input [15:0] CONV_WAIT_LOW_REG;
input [15:0] CONV_WAIT_HIGH_REG;
output CONV_OUT;
reg CONV_OUT;
input [1:0] CONV_SEL;
input NON_CONT;
input RESET_CONV;
input [7:0] CLKDELAY_AROUND_CONV;

reg [23:0] conv_count_low;
reg [23:0] conv_count_high;
reg [15:0] conv_wait_count_low;
reg [15:0] conv_wait_count_high;
reg oscillating_conv;

reg	[2:0]	FSMn, FSM;

output PAUSE_FOR_CONV;
reg PAUSE_FOR_CONV;

reg		oscillating_conv_i;
reg		decr_conv_count_low;
reg		decr_conv_count_high;
reg		decr_conv_wait_count_low;
reg		decr_conv_wait_count_high;
reg		load_conv_count_low;
reg		load_conv_count_high;
reg		load_conv_wait_count_low;
reg		load_conv_wait_count_high;

parameter 	IDLE = 4'h0,
				CONV_LOW_ST = 4'h1,
				CONV_HIGH_ST = 4'h2,
				CONV_WAIT_LOW_ST = 4'h3,
				CONV_WAIT_HIGH_ST = 4'h4;				
				
wire reset_conv_internal;
assign reset_conv_internal = FPGA_RESET && RESET_CONV;


always @(CONV_LOW_REG or CONV_HIGH_REG or conv_count_low or conv_count_high or reset_conv_internal or CONV_OUT or CLKDELAY_AROUND_CONV)
	begin
		if(reset_conv_internal==1'b0)
			begin
				PAUSE_FOR_CONV=1'b0;
			end
		else
			begin
				if((((conv_count_low > CONV_LOW_REG-CLKDELAY_AROUND_CONV) || (conv_count_low < CLKDELAY_AROUND_CONV)) && CONV_OUT==1'b0) || (((conv_count_high > CONV_HIGH_REG-CLKDELAY_AROUND_CONV) || (conv_count_high < CLKDELAY_AROUND_CONV)) && CONV_OUT==1'b1) )
					begin
						PAUSE_FOR_CONV=1'b1;
					end
				else
					begin
						PAUSE_FOR_CONV=1'b0;
					end
			end
	end

always @(posedge DDC_CLK or negedge reset_conv_internal)
  begin

  if(reset_conv_internal==1'b0)
		begin
			conv_count_low[23:0] = 24'h00_0640;
			conv_count_high[23:0] = 24'h00_0640;
			conv_wait_count_low[15:0] = 16'h1004;
			conv_wait_count_high[15:0] = 16'h1004;
			CONV_OUT = 1'b0;
			FSM = IDLE;
		end
	else
		begin
			FSM = FSMn;
			CONV_OUT = oscillating_conv_i;
			if(decr_conv_count_low==1) conv_count_low = conv_count_low-1;
			if(decr_conv_count_high==1) conv_count_high = conv_count_high-1;
			if(decr_conv_wait_count_low==1) conv_wait_count_low = conv_wait_count_low-1;
			if(decr_conv_wait_count_high==1) conv_wait_count_high = conv_wait_count_high-1;
			if(load_conv_count_low==1) conv_count_low = CONV_LOW_REG;
			if(load_conv_count_high==1) conv_count_high = CONV_HIGH_REG;
			if(load_conv_wait_count_low==1) conv_wait_count_low = CONV_WAIT_LOW_REG;
			if(load_conv_wait_count_high==1) conv_wait_count_high = CONV_WAIT_HIGH_REG;
		end

  end
  
  
  
always @( 	FSM or
				conv_count_low or
				conv_count_high or
				conv_wait_count_low or
				conv_wait_count_high or
				NON_CONT or
				CONV_SEL
				)
	begin
		FSMn=FSM;
		oscillating_conv_i=0;
		decr_conv_count_low=0;
		decr_conv_count_high=0;
		decr_conv_wait_count_low=0;
		decr_conv_wait_count_high=0;
		load_conv_count_low=0;
		load_conv_count_high=0;
		load_conv_wait_count_low=0;
		load_conv_wait_count_high=0;
		
		case(FSM)
			IDLE:
				begin
					if(CONV_SEL[1]==1)
						begin
							FSMn=IDLE;
							oscillating_conv_i=CONV_SEL[0];
						end
					else //CONV_SEL[1] == 0
						begin
							if(NON_CONT==0) //Continuous mode, nothing special
								begin
									FSMn=CONV_LOW_ST;
									oscillating_conv_i=0;
									load_conv_count_low=1;
								end
							else //NON_CONT==1, non continuous operation mode
								begin
									FSMn=CONV_HIGH_ST;
									oscillating_conv_i=1;
									load_conv_count_high=1;
								end
						end //CONV_SEL[1]==1
				end //IDLE
				
			CONV_LOW_ST:
				begin
					if(CONV_SEL[1]==1) //need to be able to get back to IDLE state.  do it here.
						begin
							FSMn=IDLE;
							oscillating_conv_i=CONV_SEL[0];
						end
					else
						begin
							if(|conv_count_low[23:0] == 1) //more to go
								begin
									FSMn = CONV_LOW_ST;
									decr_conv_count_low=1;
									oscillating_conv_i=0;
								end
							else  //otherwise just go on.
								begin
									FSMn = CONV_HIGH_ST;
									oscillating_conv_i=1;
									load_conv_count_high=1;
								end
						end
				end // CONV_LOW_ST
				
			CONV_HIGH_ST:
				begin
					if(CONV_SEL[1]==1) //need to be able to get back to IDLE state.  do it here.
						begin
							FSMn=IDLE;
							oscillating_conv_i=CONV_SEL[0];
						end
					else
						begin
							if(|conv_count_high[23:0] == 1) //more to go
								begin
									FSMn = CONV_HIGH_ST;
									decr_conv_count_high=1;
									oscillating_conv_i=1;
								end
							else
								begin
									if(NON_CONT==0)
										begin
											FSMn = CONV_LOW_ST;
											oscillating_conv_i=0;
											load_conv_count_low=1;
										end
									else //NON_CONT == 1
										begin
											FSMn = CONV_WAIT_LOW_ST;
											oscillating_conv_i=0;
											load_conv_wait_count_low=1;
										end
								end
						end
				end // CONV_HIGH_ST
				
			CONV_WAIT_LOW_ST:
				begin
					if(CONV_SEL[1]==1) //need to be able to get back to IDLE state.  do it here.
						begin
							FSMn=IDLE;
							oscillating_conv_i=CONV_SEL[0];
						end
					else
						begin
							if(|conv_wait_count_low[15:0] == 1) //more to go
								begin
									FSMn = CONV_WAIT_LOW_ST;
									decr_conv_wait_count_low=1;
									oscillating_conv_i=0;
								end
							else
								begin
									FSMn = CONV_WAIT_HIGH_ST;
									oscillating_conv_i=1;
									load_conv_wait_count_high=1;
								end
						end
				end // CONV_WAIT_LOW_ST
				
			CONV_WAIT_HIGH_ST:
				begin
					if(CONV_SEL[1]==1) //need to be able to get back to IDLE state.  do it here.
						begin
							FSMn=IDLE;
							oscillating_conv_i=CONV_SEL[0];
						end
					else
						begin
							if(|conv_wait_count_high[15:0] == 1) //more to go
								begin
									FSMn = CONV_WAIT_HIGH_ST;
									decr_conv_wait_count_high=1;
									oscillating_conv_i=1;
								end
							else
								begin
									FSMn = CONV_LOW_ST;
									oscillating_conv_i=0;
									load_conv_count_low=1;
								end
						end
				end // CONV_WAIT_HIGH_ST
			
			default:
				begin
					FSMn=IDLE;
				end
			
		endcase //FSM
	end

//assign CONV_OUT = (CONV_SEL[1]==1'b0) ? oscillating_conv : (CONV_SEL[0]==1'b0) ? 1'b0 : 1'b1;
						//CONV SEL [ manual control bit (if 1, then manual control) : manual state ]

endmodule
