`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:    10:55:25 02/09/06
// Design Name:    
// Module Name:    DDC_ACQ_CONTROL
// Project Name:   
// Target Device:  
// Tool versions:  
// Description:
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////
module DDC_ACQ_CONTROL(	FPGA_CLK,
								//DDC_CLK,
								FPGA_RESET,

								DVALIDn,
								DCLK,

								DVALIDS_TO_IGNORE,
								DVALIDS_TO_CAPTURE,
								START_CONVERSIONS,

								TRIGGER,
								SR_EMPTY_GATED,
								ACK,
								DONE,

								RESERVE_RAM_FOR_WRITING,
								RESET_ADDRESS_COUNTERS,

								HARDWARE_TRIGGER_EN,
								HARDWARE_TRIGGER,
								
								FIRST_READINGS,
								RESET_AVGRAM_ADDR,
								
								STATE_MACHINE_ACTIVE
								);
input FPGA_CLK;
//input DDC_CLK;
input FPGA_RESET;

input DVALIDn;
output DCLK;
reg DCLK;

input [7:0] DVALIDS_TO_IGNORE;	//[7:0]
input [23:0] DVALIDS_TO_CAPTURE;	//[23:0]
input START_CONVERSIONS;

output TRIGGER;
reg TRIGGER;
input SR_EMPTY_GATED;
input ACK;
output DONE;
reg DONE;

output RESERVE_RAM_FOR_WRITING;
reg RESERVE_RAM_FOR_WRITING;

output RESET_ADDRESS_COUNTERS;
reg RESET_ADDRESS_COUNTERS;

input HARDWARE_TRIGGER_EN;
input HARDWARE_TRIGGER;

output [1:0] FIRST_READINGS;
reg [1:0] FIRST_READINGS;

output RESET_AVGRAM_ADDR;
reg RESET_AVGRAM_ADDR;

output STATE_MACHINE_ACTIVE;
wire STATE_MACHINE_ACTIVE;

parameter	STATE00 = 4'b0000,
				STATE01 = 4'b0001,
				STATE02 = 4'b0010,
				STATE03 = 4'b0011,
				STATE04 = 4'b0100,
				STATE05 = 4'b0101,
				STATE06 = 4'b0110,
				STATE07 = 4'b0111,
				STATE08 = 4'b1000,
				STATE09 = 4'b1001;

reg [3:0] FSM, FSMn;

reg [7:0] dvalids_to_ignore_reg;
reg [23:0] dvalids_to_capture_reg;

reg [7:0] dclk_wait_counter;
reg [7:0] dclk_high_counter;

reg trigger_readback_i;
reg done_bit_i;
reg load_ignore_counter;
reg decrement_ignore_counter;
reg load_dclk_wait_counter;
reg load_dclk_high_counter;
reg load_capture_counter;
reg decrement_dclk_wait_counter;
reg decrement_dclk_high_counter;
reg decrement_capture_counter;
reg dclk_i;
reg reserve_ram_for_writing_i;
reg reset_address_counters_i;

reg reset_first_readings_i;
reg incr_first_readings_i;

reg reset_avgram_addr_i;


assign STATE_MACHINE_ACTIVE=!(FSM==STATE00);

always @(posedge FPGA_CLK or negedge FPGA_RESET)
	begin
		if(FPGA_RESET == 1'b0)
   		begin
				TRIGGER = 1'b0;
				DONE = 1'b0;
				DCLK = 1'b0;
				dvalids_to_ignore_reg[7:0]=8'b00000000;
				dclk_wait_counter[7:0]=8'b00000000;
				dclk_high_counter[7:0]=8'b00000000;
				dvalids_to_capture_reg[23:0]=24'b0000_0000__0000_0000__0000_0000;
   			FSM = STATE00;
				RESERVE_RAM_FOR_WRITING=1'b0;
				RESET_ADDRESS_COUNTERS=1'b0;
				FIRST_READINGS[1:0]=2'b00;
				RESET_AVGRAM_ADDR=1'b0;
			end
		else
			begin
				TRIGGER = trigger_readback_i;
				DONE = done_bit_i;
				DCLK = dclk_i;
				FSM = FSMn;
				if(load_ignore_counter==1'b1) dvalids_to_ignore_reg[7:0] = DVALIDS_TO_IGNORE[7:0];
				if(decrement_ignore_counter==1'b1) dvalids_to_ignore_reg[7:0] = dvalids_to_ignore_reg[7:0] - 8'h01;
				if(load_dclk_wait_counter==1'b1) dclk_wait_counter[7:0] = 8'h80;
				if(decrement_dclk_wait_counter==1'b1) dclk_wait_counter[7:0] = dclk_wait_counter[7:0] - 8'h01;
				if(load_dclk_high_counter==1'b1) dclk_high_counter[7:0] = 8'h10;
				if(decrement_dclk_high_counter==1'b1) dclk_high_counter[7:0] = dclk_high_counter[7:0] - 8'h01;
				if(load_capture_counter==1'b1) dvalids_to_capture_reg[23:0] = DVALIDS_TO_CAPTURE[23:0];
				if(decrement_capture_counter==1'b1) dvalids_to_capture_reg[23:0] = dvalids_to_capture_reg[23:0] - 24'h000001;
				RESERVE_RAM_FOR_WRITING=reserve_ram_for_writing_i;
				RESET_ADDRESS_COUNTERS=reset_address_counters_i;
				if(reset_first_readings_i==1'b1) FIRST_READINGS[1:0] = 2'b00;
				if(incr_first_readings_i==1'b1)
					begin
						if(FIRST_READINGS[1]==1'b0)
							begin
								FIRST_READINGS[1:0] = FIRST_READINGS[1:0]+2'b01;
							end
						else
							begin
								FIRST_READINGS[0] = !FIRST_READINGS[0];
							end
					end
				RESET_AVGRAM_ADDR = reset_avgram_addr_i;
			end
	end

always @(	FSM or
				START_CONVERSIONS or
				dvalids_to_ignore_reg or
				dvalids_to_capture_reg or
				DVALIDn or
				dclk_wait_counter or
				dclk_high_counter or
				ACK or
				SR_EMPTY_GATED	or
				HARDWARE_TRIGGER_EN or
				HARDWARE_TRIGGER
				)
	begin
		load_ignore_counter=1'b0;
		decrement_ignore_counter=1'b0;

		load_dclk_wait_counter=1'b0;
		decrement_dclk_wait_counter=1'b0;

		load_dclk_high_counter=1'b0;
		decrement_dclk_high_counter=1'b0;

		load_capture_counter=1'b0;
		decrement_capture_counter=1'b0;

		trigger_readback_i=1'b0;
		done_bit_i=1'b0;
		
		dclk_i=1'b0;
		
		reserve_ram_for_writing_i=1'b1;
		reset_address_counters_i=1'b0;
		
		reset_first_readings_i=1'b0;
		incr_first_readings_i=1'b0;
		
		reset_avgram_addr_i=1'b0;

		case(FSM)
			STATE00:  //idle state
				begin
					if( START_CONVERSIONS==1'b1 && ((HARDWARE_TRIGGER_EN==1'b1 && HARDWARE_TRIGGER==1'b1) || (HARDWARE_TRIGGER_EN==1'b0)) )
						begin
							load_capture_counter=1'b1;
							load_ignore_counter=1'b1;
							reset_address_counters_i=1'b1;
							reset_first_readings_i=1'b1;
							FSMn=STATE01;
						end
					else
						begin
							FSMn=STATE00;
							reserve_ram_for_writing_i=1'b0;
						end
				end

			STATE01:  //reload all registers, direct to appropriate next state for reads/ignores/done
				begin
					case( {|dvalids_to_ignore_reg[7:0],|dvalids_to_capture_reg[23:0]} )
						2'b10:  //10 and 11 are the same, since both involve going to state02
							begin
								FSMn = STATE02;
							end
						2'b11:
							begin
								FSMn = STATE02;
							end

						2'b01:  //01 goes to state05, skipping the ignores but doing the reads
							begin
								FSMn = STATE05;
							end

						2'b00:  //00 goes to state08, skipping the ignores and reads
							begin
								FSMn = STATE08;
							end

						default:	 //this goes to state08, skipping the ignores and reads
							begin
								FSMn = STATE08;
							end
					endcase
				end

			STATE02:	 //ignore loop - wait for next dvalid
				begin
					if(DVALIDn==1'b1)
						begin
							FSMn = STATE02;
						end
					else
						begin
							decrement_ignore_counter = 1'b1;
							load_dclk_wait_counter = 1'b1;
							FSMn = STATE03;
						end
				end

			STATE03:  //wait cycle(s)
				begin
					if(|dclk_wait_counter[7:0]==1'b1)
						begin
							decrement_dclk_wait_counter=1'b1;
							FSMn = STATE03;
						end
					else
						begin
							load_dclk_high_counter = 1'b1;
							FSMn = STATE04;
							dclk_i = 1'b1;
						end
				end

			STATE04:
				begin  //dclk high cycle(s)
					if(|dclk_high_counter[7:0]==1'b1)
						begin
						   dclk_i = 1'b1;
							decrement_dclk_high_counter=1'b1;
							FSMn = STATE04;
						end
					else if(DVALIDn==1'b0)
					   begin
					      FSMn = STATE04;
					   end
					else
						begin
							case( {|dvalids_to_ignore_reg[7:0],|dvalids_to_capture_reg[23:0]} )
								2'b11:
									begin
										FSMn = STATE02;  //go back to waiting for more dvalids to ignore
									end																  
								2'b10:
									begin
										FSMn = STATE02;  //go back to waiting for more dvalids to ignore
									end
								2'b01:
									begin
										FSMn = STATE05;  //go back to waiting for more dvalids to read
									end
								2'b00:
									begin
										FSMn = STATE08;  //go to done
									end
								default:
									begin
										FSMn = STATE08;  //go to done
									end
							endcase
						end
				end

			STATE05:  //wait for dvalid, start read cycle when you get it.
				begin
					reset_avgram_addr_i=1'b1;
					if(DVALIDn==1'b1)
						begin
							FSMn = STATE05;
						end
					else
						begin
							decrement_capture_counter = 1'b1;
							FSMn = STATE06;
						end
				end

			STATE06:	 //trigger the capture, also should clear the ACK flag
				begin
					trigger_readback_i=1'b1;
					FSMn = STATE07;
				end

			STATE07:  //wait for ack
				begin
					if(ACK==1'b1)
						begin
							FSMn = STATE04;  //go on into next dvalid state
							incr_first_readings_i=1'b1; //incr by 1 if FIRST_READINGS[1:0] < 2'b10
						end
					else
						begin
							trigger_readback_i=1'b1;
							FSMn = STATE07;
						end
				end

			STATE08:  //finish stuff, send register data back, etc...
				begin
				   if(SR_EMPTY_GATED==1'b0)  //if not empty, sit here and spin until it is.
				      begin
				         FSMn = STATE08;
				      end
				   else
				      begin
				         done_bit_i=1'b1;
					      reserve_ram_for_writing_i=1'b0;
					      FSMn = STATE09;
					   end
				end

			STATE09:	 //wait until 
				begin
					if(START_CONVERSIONS==1'b1)
						begin 
						   done_bit_i=1'b1;
							reserve_ram_for_writing_i=1'b0;
							FSMn = STATE09;
						end
					else
						begin
							reserve_ram_for_writing_i=1'b0;
							FSMn = STATE00;
						end
				end

			default:
				begin
					reserve_ram_for_writing_i=1'b0;
					FSMn=STATE00;
				end
		endcase
	end

endmodule
