`timescale 1ns / 100ps

module DDC_DATA_FIFO(	FPGA_CLK,
								FPGA_RESET,

								//DDC_DATA_CAPTURE interface
								DATA_FROM_DDC, //20 bits, sometimes the [19:16] are ignored if in 16 bit mode.
								A0_OR_B1_DATA,
								
								DATA_WIDTH, //0 = 16 bit, 1 = 20 bit (broken up into 16 and 4)
								LATCH_DATA_FROM_DDC, //DDC_DATA_CAPTURE sets this to 1 when new data is available for latching
								DATA_FROM_DDC_PROCESSED, //we set this when we're done parsing and loading ddc data into the fifo

								//RAM_TO_USB_FIFO interface
								DATA_FROM_FIFO_TO_RAM, //always 16 bits.

								//RESERVE_RAM_FOR_WRITING,
								SR_EMPTY_GATED,
								RAM_WRITE_DATA_PROCESSED,
								READY_TO_WRITE,
								
								DDC_FIFO_WRITE_DATA_AVAILABLE, //we set this when we have data for the USB_IO block to send back
								RAM_WRITE_DATA_REQUESTED  //USB_IO sets this to 1 when it is ready for our data
								//DDC_FIFO_WRITE_DATA_LOADED  //we set this when the data for USB_IO is on the DATA_FROM_FIFO_TO_RAM lines
								);

//////////// DECLARATIONS ////////////
input FPGA_CLK, FPGA_RESET; 

input [19:0] DATA_FROM_DDC;

input A0_OR_B1_DATA;

input DATA_WIDTH;

input LATCH_DATA_FROM_DDC;

output DATA_FROM_DDC_PROCESSED;
reg DATA_FROM_DDC_PROCESSED;
reg DATA_FROM_DDC_PROCESSED_i;

output [15:0] DATA_FROM_FIFO_TO_RAM;
wire [15:0] DATA_FROM_FIFO_TO_RAM;

//input RESERVE_RAM_FOR_WRITING;

output DDC_FIFO_WRITE_DATA_AVAILABLE;
wire DDC_FIFO_WRITE_DATA_AVAILABLE;

input RAM_WRITE_DATA_REQUESTED;
output SR_EMPTY_GATED;
reg SR_EMPTY_GATED;

//output DDC_FIFO_WRITE_DATA_LOADED;
//reg DDC_FIFO_WRITE_DATA_LOADED;
//reg ddc_fifo_write_data_loaded_i;

input RAM_WRITE_DATA_PROCESSED;
output READY_TO_WRITE;
reg READY_TO_WRITE;

//reg ram_write_data_processed_gated; //to sync with clock.

wire sr_full;

wire sr_empty;

reg increment_fpga_fifo_out_addr_i;
reg [3:1] fifo_address_in;
reg [3:0] fifo_address_out;

//reg ram_write_data_requested_gated;
//reg sr_full_gated;
reg ready_to_write_i;

reg [2:0] FSM, FSMn;
reg [1:0] FSM2, FSM2n;



//////////// PARAMETERS ////////////

parameter	IDLE = 3'b000,
				INCREMENT_FGPA_FIFO_IN_ADDRESS_AFTER_LOAD = 3'b001,
				LATCHED_DATA_NOW_WAIT_FOR_ACK = 3'b010,
				WAIT_AFTER_READ_BEFORE_IDLE = 3'b011;

parameter 	IDLE_2=2'b00,
				STORE_DATA_2=2'b01,
				WAIT_FOR_ACK_2=2'b10;



//////////// FPGA_FIFO SHIFT REGISTER ////////////

reg [15:0] fpga_fifo_full_empty;  //0 indicates empty, 1 indicates full.  all-or indicates all empty, all-and indicates all full
wire [15:0] fpga_fifo;

reg [15:0] fpga_fifo_0000;
reg [15:0] fpga_fifo_0001;
reg [15:0] fpga_fifo_0010;
reg [15:0] fpga_fifo_0011;
reg [15:0] fpga_fifo_0100;
reg [15:0] fpga_fifo_0101;
reg [15:0] fpga_fifo_0110;
reg [15:0] fpga_fifo_0111;
reg [15:0] fpga_fifo_1000;
reg [15:0] fpga_fifo_1001;
reg [15:0] fpga_fifo_1010;
reg [15:0] fpga_fifo_1011;
reg [15:0] fpga_fifo_1100;
reg [15:0] fpga_fifo_1101;
reg [15:0] fpga_fifo_1110;
reg [15:0] fpga_fifo_1111;
reg latch_data16_0_i;
reg latch_data16_2_i;
reg latch_data16_4_i;
reg latch_data16_6_i;
reg latch_data16_8_i;
reg latch_data16_A_i;
reg latch_data16_C_i;
reg latch_data16_E_i;
reg latch_data20_0_i;
reg latch_data20_2_i;
reg latch_data20_4_i;
reg latch_data20_6_i;
reg latch_data20_8_i;
reg latch_data20_A_i;
reg latch_data20_C_i;
reg latch_data20_E_i;

assign fpga_fifo[15:0] = (fifo_address_out[3:0]==4'b0000)?fpga_fifo_0000[15:0]: (fifo_address_out[3:0]==4'b0001)?fpga_fifo_0001[15:0]: (fifo_address_out[3:0]==4'b0010)?fpga_fifo_0010[15:0]: (fifo_address_out[3:0]==4'b0011)?fpga_fifo_0011[15:0]: (fifo_address_out[3:0]==4'b0100)?fpga_fifo_0100[15:0]: (fifo_address_out[3:0]==4'b0101)?fpga_fifo_0101[15:0]: (fifo_address_out[3:0]==4'b0110)?fpga_fifo_0110[15:0]: (fifo_address_out[3:0]==4'b0111)?fpga_fifo_0111[15:0]: (fifo_address_out[3:0]==4'b1000)?fpga_fifo_1000[15:0]: (fifo_address_out[3:0]==4'b1001)?fpga_fifo_1001[15:0]: (fifo_address_out[3:0]==4'b1010)?fpga_fifo_1010[15:0]: (fifo_address_out[3:0]==4'b1011)?fpga_fifo_1011[15:0]: (fifo_address_out[3:0]==4'b1100)?fpga_fifo_1100[15:0]: (fifo_address_out[3:0]==4'b1101)?fpga_fifo_1101[15:0]: (fifo_address_out[3:0]==4'b1110)?fpga_fifo_1110[15:0]: /*fifo_address_in[3:0]==4'b1111?*/ fpga_fifo_1111[15:0];
assign DATA_FROM_FIFO_TO_RAM[15:0] = fpga_fifo[15:0];

always @(posedge FPGA_CLK or negedge FPGA_RESET)
  begin
  if(FPGA_RESET==1'b0)
    begin
	 fpga_fifo_0000[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_0001[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_0010[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_0011[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_0100[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_0101[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_0110[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_0111[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1000[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1001[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1010[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1011[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1100[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1101[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1110[15:0]<=16'b0000_0000__0000_0000;
	 fpga_fifo_1111[15:0]<=16'b0000_0000__0000_0000;
	 fifo_address_in[3:1]<=3'b000;
	 fifo_address_out[3:0]<=4'h0;
	 fpga_fifo_full_empty[15:0]<=16'b0000_0000__0000_0000;
    end
  else
	 begin
	 	if(increment_fpga_fifo_out_addr_i==1'b1)
		   begin
			if(fifo_address_out[3:0]==4'b0000) fpga_fifo_full_empty[0]<=1'b0;
			if(fifo_address_out[3:0]==4'b0001) fpga_fifo_full_empty[1]<=1'b0;
			if(fifo_address_out[3:0]==4'b0010) fpga_fifo_full_empty[2]<=1'b0;
			if(fifo_address_out[3:0]==4'b0011) fpga_fifo_full_empty[3]<=1'b0;
			if(fifo_address_out[3:0]==4'b0100) fpga_fifo_full_empty[4]<=1'b0;
			if(fifo_address_out[3:0]==4'b0101) fpga_fifo_full_empty[5]<=1'b0;
			if(fifo_address_out[3:0]==4'b0110) fpga_fifo_full_empty[6]<=1'b0;
			if(fifo_address_out[3:0]==4'b0111) fpga_fifo_full_empty[7]<=1'b0;
			if(fifo_address_out[3:0]==4'b1000) fpga_fifo_full_empty[8]<=1'b0;
			if(fifo_address_out[3:0]==4'b1001) fpga_fifo_full_empty[9]<=1'b0;
			if(fifo_address_out[3:0]==4'b1010) fpga_fifo_full_empty[10]<=1'b0;
			if(fifo_address_out[3:0]==4'b1011) fpga_fifo_full_empty[11]<=1'b0;
			if(fifo_address_out[3:0]==4'b1100) fpga_fifo_full_empty[12]<=1'b0;
			if(fifo_address_out[3:0]==4'b1101) fpga_fifo_full_empty[13]<=1'b0;
			if(fifo_address_out[3:0]==4'b1110) fpga_fifo_full_empty[14]<=1'b0;
			if(fifo_address_out[3:0]==4'b1111) fpga_fifo_full_empty[15]<=1'b0;
			fifo_address_out[3:0] <= fifo_address_out[3:0] + 4'h1;
			end
				if(latch_data16_0_i==1'b1)
					begin
					fpga_fifo_0000[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_0001[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[1:0]<=2'b11;
					fifo_address_in[3:1]<=3'b001;
					end

				if(latch_data16_2_i==1'b1)
					begin
					fpga_fifo_0010[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_0011[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[3:2]<=2'b11;
					fifo_address_in[3:1]<=3'b010;
					end

				if(latch_data16_4_i==1'b1)
					begin
					fpga_fifo_0100[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_0101[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[5:4]<=2'b11;
					fifo_address_in[3:1]<=3'b011;
					end

				if(latch_data16_6_i==1'b1)
					begin
					fpga_fifo_0110[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_0111[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[7:6]<=2'b11;
					fifo_address_in[3:1]<=3'b100;
					end

				if(latch_data16_8_i==1'b1)
					begin
					fpga_fifo_1000[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_1001[15:0]<= DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[9:8]<=2'b11;
					fifo_address_in[3:1]<=3'b101;
					end

				if(latch_data16_A_i==1'b1)
					begin
					fpga_fifo_1010[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_1011[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[11:10]<=2'b11;
					fifo_address_in[3:1]<=3'b110; 
					end

				if(latch_data16_C_i==1'b1)
					begin
					fpga_fifo_1100[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_1101[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[13:12]<=2'b11;
					fifo_address_in[3:1]<=3'b111;
					end

				if(latch_data16_E_i==1'b1)
					begin
					fpga_fifo_1110[15:0]<={A0_OR_B1_DATA,15'h7FFF};
					fpga_fifo_1111[15:0]<= DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[15:14]<=2'b11;
					fifo_address_in[3:1]<=3'b000;
					end

				if(latch_data20_0_i==1'b1)
					begin
					fpga_fifo_0000[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_0001[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[1:0]<=2'b11;
					fifo_address_in[3:1]<=3'b001;
					end

				if(latch_data20_2_i==1'b1)
					begin
					fpga_fifo_0010[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_0011[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[3:2]<=2'b11;
					fifo_address_in[3:1]<=3'b010;
					end

				if(latch_data20_4_i==1'b1)
					begin
					fpga_fifo_0100[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_0101[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[5:4]<=2'b11;
					fifo_address_in[3:1]<=3'b011;
					end	

				if(latch_data20_6_i==1'b1)
					begin
					fpga_fifo_0110[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_0111[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[7:6]<=2'b11;
					fifo_address_in[3:1]<=3'b100;
					end

				if(latch_data20_8_i==1'b1)
					begin
					fpga_fifo_1000[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_1001[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[9:8]<=2'b11;
					fifo_address_in[3:1]<=3'b101;
					end

				if(latch_data20_A_i==1'b1)
					begin
					fpga_fifo_1010[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_1011[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[11:10]<=2'b11;
					fifo_address_in[3:1]<=3'b110;
					end

				if(latch_data20_C_i==1'b1)
					begin
					fpga_fifo_1100[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_1101[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[13:12]<=2'b11;
					fifo_address_in[3:1]<=3'b111;
					end

				if(latch_data20_E_i==1'b1)
					begin
					fpga_fifo_1110[15:0]<={{A0_OR_B1_DATA,11'h000},DATA_FROM_DDC[19:16]};
					fpga_fifo_1111[15:0]<=DATA_FROM_DDC[15:0];
					fpga_fifo_full_empty[15:14]<=2'b11;
					fifo_address_in[3:1]<=3'b000;
					end
	 end
  end




//////////// ASSIGNMENTS ////////////
assign sr_empty=~(|fpga_fifo_full_empty[15:0]);
assign sr_full=(fpga_fifo_full_empty[15]||fpga_fifo_full_empty[14]) && (fpga_fifo_full_empty[13]||fpga_fifo_full_empty[12]) && (fpga_fifo_full_empty[11]||fpga_fifo_full_empty[10]) && (fpga_fifo_full_empty[9]||fpga_fifo_full_empty[8]) && (fpga_fifo_full_empty[7]||fpga_fifo_full_empty[6]) && (fpga_fifo_full_empty[5]||fpga_fifo_full_empty[4]) && (fpga_fifo_full_empty[3]||fpga_fifo_full_empty[2]) && (fpga_fifo_full_empty[1]||fpga_fifo_full_empty[0]);
assign DDC_FIFO_WRITE_DATA_AVAILABLE = !SR_EMPTY_GATED;



//////////// ALWAYS BLOCKS ////////////

always @(posedge FPGA_CLK or negedge FPGA_RESET)
  begin
  if(FPGA_RESET==1'b0)
    begin
    FSM<=IDLE;
	 FSM2<=IDLE_2;	
    //DDC_FIFO_WRITE_DATA_LOADED=1'b0;
    
    DATA_FROM_DDC_PROCESSED<=1'b0;
    //ram_write_data_processed_gated=1'b0;
	 //ram_write_data_requested_gated=1'b0;
	 SR_EMPTY_GATED<=1'b1;
	 //sr_full_gated=1'b0;
	 READY_TO_WRITE<=1'b1;
    end
  else
    begin
    FSM<=FSMn;
	 FSM2<=FSM2n;
    //DDC_FIFO_WRITE_DATA_LOADED=ddc_fifo_write_data_loaded_i;
    
    DATA_FROM_DDC_PROCESSED<=DATA_FROM_DDC_PROCESSED_i;
    //ram_write_data_processed_gated=RAM_WRITE_DATA_PROCESSED;
	 //ram_write_data_requested_gated=RAM_WRITE_DATA_REQUESTED;
	 SR_EMPTY_GATED<=sr_empty;
	 //sr_full_gated=sr_full;
	 READY_TO_WRITE<=ready_to_write_i;
    end
  end

//////////// STATE MACHINES ////////////





always @(	FSM or
				//ram_write_data_requested_gated or
				RAM_WRITE_DATA_REQUESTED or
				RAM_WRITE_DATA_PROCESSED or
				SR_EMPTY_GATED
				)
  begin
  FSMn<=FSM;
  //ddc_fifo_write_data_loaded_i=1'b0;
  increment_fpga_fifo_out_addr_i<=1'b0;
  ready_to_write_i<=1'b1;
  
  case(FSM)
    IDLE:
      begin

	   if(RAM_WRITE_DATA_REQUESTED==1'b1 && SR_EMPTY_GATED==1'b0) //data is requested from memory_con, shift reg not empty.
	     begin
	     FSMn<=LATCHED_DATA_NOW_WAIT_FOR_ACK;
	     ready_to_write_i<=1'b0;
	     end
	   else
	     begin
	     FSMn<=IDLE;
	     end 
	   end

    LATCHED_DATA_NOW_WAIT_FOR_ACK:   //INCREMENT_FGPA_FIFO_OUT_ADDRESS_AFTER_WRITE
      begin
	   if(RAM_WRITE_DATA_PROCESSED==1'b1) 
	     begin
	     increment_fpga_fifo_out_addr_i<=1'b1;
	     FSMn<=IDLE;
	     end
	   else   //still processing data on memory_con side
	     begin
	     FSMn<=LATCHED_DATA_NOW_WAIT_FOR_ACK;
	     ready_to_write_i<=1'b0;
	     end
	   end

    default:
      begin
	 FSMn<=IDLE;
	 end

    endcase
  end	 //FSM







always @(	FSM2 or
				LATCH_DATA_FROM_DDC or
				fifo_address_in or
				fpga_fifo_full_empty or
				DATA_WIDTH or
				sr_full
				)
	begin
		FSM2n<=FSM2;
		latch_data16_0_i<=1'b0;
		latch_data16_2_i<=1'b0;
		latch_data16_4_i<=1'b0;
		latch_data16_6_i<=1'b0;
		latch_data16_8_i<=1'b0;
		latch_data16_A_i<=1'b0;
		latch_data16_C_i<=1'b0;
		latch_data16_E_i<=1'b0;
		latch_data20_0_i<=1'b0;
		latch_data20_2_i<=1'b0;
		latch_data20_4_i<=1'b0;
		latch_data20_6_i<=1'b0;
		latch_data20_8_i<=1'b0;
		latch_data20_A_i<=1'b0;
		latch_data20_C_i<=1'b0;
		latch_data20_E_i<=1'b0;
		DATA_FROM_DDC_PROCESSED_i<=1'b0;

		case(FSM2)
			IDLE_2:
			begin
				if(LATCH_DATA_FROM_DDC==1'b1 && sr_full==1'b0)
					begin
						FSM2n<=STORE_DATA_2;
	 					//DATA_FROM_DDC_PROCESSED_i=1'b1;
					end
				else
					begin
						FSM2n<=IDLE_2;
					end
			end

			STORE_DATA_2:
			begin
				if(DATA_WIDTH==1'b0)	//16 bit
					begin
						case(fifo_address_in[3:1])
							3'b000:
							begin
								if(fpga_fifo_full_empty[1:0]==2'b00) latch_data16_0_i<=1'b1;
							end
							3'b001:
							begin
								if(fpga_fifo_full_empty[3:2]==2'b00) latch_data16_2_i<=1'b1;
							end
							3'b010:
							begin
								if(fpga_fifo_full_empty[5:4]==2'b00) latch_data16_4_i<=1'b1;
							end
							3'b011:
							begin
								if(fpga_fifo_full_empty[7:6]==2'b00) latch_data16_6_i<=1'b1;
							end
							3'b100:
							begin
								if(fpga_fifo_full_empty[9:8]==2'b00) latch_data16_8_i<=1'b1;
							end
							3'b101:
							begin
								if(fpga_fifo_full_empty[11:10]==2'b00) latch_data16_A_i<=1'b1;
							end
							3'b110:
							begin
								if(fpga_fifo_full_empty[13:12]==2'b00) latch_data16_C_i<=1'b1;
							end
							3'b111:
							begin
								if(fpga_fifo_full_empty[15:14]==2'b00) latch_data16_E_i<=1'b1;
							end
						endcase
					end
				else //(DATA_WIDTH==1'b1) 20 bit
					begin
						case(fifo_address_in[3:1])
							3'b000:
							begin
								if(fpga_fifo_full_empty[1:0]==2'b00) latch_data20_0_i<=1'b1;
							end
							3'b001:
							begin
								if(fpga_fifo_full_empty[3:2]==2'b00) latch_data20_2_i<=1'b1;
							end
							3'b010:
							begin
								if(fpga_fifo_full_empty[5:4]==2'b00) latch_data20_4_i<=1'b1;
							end
							3'b011:
							begin
								if(fpga_fifo_full_empty[7:6]==2'b00) latch_data20_6_i<=1'b1;
							end
							3'b100:
							begin
								if(fpga_fifo_full_empty[9:8]==2'b00) latch_data20_8_i<=1'b1;
							end
							3'b101:
							begin
								if(fpga_fifo_full_empty[11:10]==2'b00) latch_data20_A_i<=1'b1;
							end
							3'b110:
							begin
								if(fpga_fifo_full_empty[13:12]==2'b00) latch_data20_C_i<=1'b1;
							end
							3'b111:
							begin
								if(fpga_fifo_full_empty[15:14]==2'b00) latch_data20_E_i<=1'b1;
							end
						endcase
					end
				DATA_FROM_DDC_PROCESSED_i<=1'b1;
				FSM2n<=WAIT_FOR_ACK_2;
			end

			WAIT_FOR_ACK_2:  //wait for latching signal to drop.
			begin
				if(LATCH_DATA_FROM_DDC==1'b0)
					begin
						FSM2n<=IDLE_2;
					end
				else
					begin
						FSM2n<=WAIT_FOR_ACK_2;
						DATA_FROM_DDC_PROCESSED_i<=1'b1;
					end
			end

			default:
			begin
				FSM2n<=IDLE_2;
			end

		endcase  
	end //FSM2








endmodule



