`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:    12:08:18 06/27/06
// Design Name:    
// Module Name:    DVALID_GENERATION
// Project Name:   
// Target Device:  
// Tool versions:  
// Description:
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////
module DVALID_GENERATION(FPGA_CLK,
								 FPGA_RESET,
								 TRIGGER, 
								 CLK_CFG, 
								 DIN_CFG,
								 CLK_CFG_HIGH,
								 CLK_CFG_LOW,
								 FORMAT,
								 DIN_CFG_HIGHER_BITS,
								 DIN_CFG_LOWER_BITS,
								 DVALID_GEN,
								 START_C,
								 IGNORES,
								 READING,
								 DONE_READ);

input FPGA_CLK;
input FPGA_RESET;
input TRIGGER;

output CLK_CFG;
reg CLK_CFG;
reg dec_low;
reg dec_hig;

output DIN_CFG;
reg DIN_CFG;

output DVALID_GEN;
reg DVALID_GEN;

input [3:0] CLK_CFG_HIGH;
input [3:0] CLK_CFG_LOW;

input FORMAT;
input [7:0] DIN_CFG_HIGHER_BITS;
input [7:0] DIN_CFG_LOWER_BITS;

output START_C;
reg START_C;

output [7:0]IGNORES;
reg [7:0]IGNORES;

output [23:0]READING;
reg [23:0]READING;

input DONE_READ;

reg [3:0] FSMn, FSM;
reg [3:0] config_CLK;
reg [3:0] count_high;
reg [3:0] count_low;
reg [3:0] COUNTER_AUX;
reg [7:0] COUNTER_AUX_READ_WRITE;
reg load_counter_high;
reg load_counter_low;
reg load_config_clk;
reg dec_config;
reg [15:0] pattern_generation;
reg TRIGGER_i;
reg dvalid_gen_control;
reg start_conv;
reg load_counter_aux;
reg dec_aux_count;
reg load_counter_aux_read_write;
reg dec_counter_aux_red_write;
reg set_up;
reg DIN_CFG_i;

 parameter	IDLE = 4'b0000,
 				WAIT = 4'b0001,
				CLK_CFG_HIGH_STATE = 4'b0011,
				CLK_CFG_LOW_STATE = 4'b0010,
				DELAY_WRITE_READ = 4'b1011,
				SET_START_CONVERSION = 4'b0110,
				WAIT_1 = 4'b0111,
				SET_DVALID = 4'b0101,
				WAIT_FOR_DONE = 4'b0100,
				DONE_START_CONVERSION = 4'b1100,
				DONE = 4'b1000,
				DELAY = 4'b1001;


always @(posedge FPGA_CLK or negedge FPGA_RESET)
	begin
		if(FPGA_RESET==1'b0)
			begin
			 	FSM <= IDLE;
				config_CLK[3:0] <= 4'h0;
				count_high[3:0] <= 4'h0;
				count_low[3:0] <= 4'h0;
				TRIGGER_i <= 1'b0;
				DVALID_GEN <= 1'b1;
				pattern_generation[15:0] <= 16'h0000;
				START_C <= 1'b0;
				COUNTER_AUX[3:0] <= 4'h0;
				COUNTER_AUX_READ_WRITE[7:0] <= 8'h0; 
				IGNORES[7:0] <= 8'hFF;
				READING[23:0] <= 24'h0;
			end
		else if(TRIGGER == 1'b1)
			begin
				FSM <= FSMn;
				if(load_config_clk == 1'b1)
					begin
						if(FORMAT ==1'b1)
							begin
								config_CLK[3:0] <= 4'hF;
							end
						else
							begin
								config_CLK[3:0] <= 4'hB;
							end
					end
				if(dec_config == 1'b1) config_CLK[3:0] <= config_CLK[3:0] - 4'b0001;
				if(load_counter_high ==1'b1) count_high <= CLK_CFG_HIGH;
				if(dec_hig ==1'b1) count_high[3:0] <= count_high[3:0] - 4'b0001;
				if(load_counter_low ==1'b1) count_low <= CLK_CFG_LOW;	
				if(dec_low ==1'b1) count_low[3:0] <= count_low[3:0] - 4'b0001;
				TRIGGER_i <= 1'b1;
				if(dvalid_gen_control == 1'b1)
					begin
						DVALID_GEN <= 1'b0;
					end
				else
					begin
						DVALID_GEN <= 1'b1;
					end
				pattern_generation[15:0] <= {DIN_CFG_HIGHER_BITS[7:0],DIN_CFG_LOWER_BITS[7:0]};
				if(start_conv == 1'b1)
					begin
						START_C <= 1'b1;
					end
				else
					begin
						START_C <= 1'b0;
					end

				if (load_counter_aux == 1'b1)
					begin
						COUNTER_AUX[3:0] <= 4'hF;
					end
				else
					begin
						COUNTER_AUX[3:0] <= COUNTER_AUX[3:0];
					end
				if(dec_aux_count ==1'b1) COUNTER_AUX[3:0] <= COUNTER_AUX[3:0] - 4'b0001;

				if(load_counter_aux_read_write == 1'b1)
					begin
						COUNTER_AUX_READ_WRITE[7:0] <= 8'hFF;
					end
				else
					begin
						COUNTER_AUX_READ_WRITE[7:0] <= COUNTER_AUX_READ_WRITE[7:0];
					end
				if(dec_counter_aux_red_write == 1'b1) COUNTER_AUX_READ_WRITE[7:0] <= COUNTER_AUX_READ_WRITE[7:0] - 8'h01;
				
				if(set_up == 1'b1)
					begin
						IGNORES[7:0] <= 8'h00;
						READING[23:0] <= 24'h000001;
					end
				else
					begin
						IGNORES[7:0] <= 8'h00;
						READING[23:0] <= 24'h000000;
					end
			end
		else
			begin 
				TRIGGER_i <= 1'b0;
			end

end



always @(	FSM or 
				TRIGGER_i or 
				config_CLK or 
				count_high or 
				count_low or 
				COUNTER_AUX or 
				DONE_READ or 
				COUNTER_AUX_READ_WRITE or
				pattern_generation or 
				FORMAT)
	begin
	dec_low <= 1'b0;
	dec_hig <= 1'b0;
	load_config_clk <= 1'b0;
	load_counter_high <= 1'b0;
	load_counter_low <= 1'b0;
	dec_config <= 1'b0;
	load_counter_aux <= 1'b0;
	dec_aux_count <= 1'b0;
	set_up <= 1'b0;
	start_conv <= 1'b0;
	dvalid_gen_control <= 1'b0;
	CLK_CFG <= 1'b0;
	DIN_CFG <= 1'b0;
	dec_aux_count <= 1'b0;
	FSMn <= FSM;

		case(FSM)
			IDLE:
				begin
					if(TRIGGER_i ==1'b1)
						begin
							FSMn <= WAIT;
							load_config_clk <=1'b1; 
						end
					else
						begin
							FSMn <= IDLE;
						end
				end
	
			WAIT:
				begin
					FSMn <= CLK_CFG_HIGH_STATE;	
					load_counter_high <= 1'b1;
					CLK_CFG <= 1'b1;
					case(config_CLK[3:0])
						4'b0000: DIN_CFG <= pattern_generation[0];
						4'b0001: DIN_CFG <= pattern_generation[1];
						4'b0010: DIN_CFG <= pattern_generation[2];
						4'b0011: DIN_CFG <= pattern_generation[3];
						4'b0100: DIN_CFG <= pattern_generation[4];
						4'b0101: DIN_CFG <= pattern_generation[5];
						4'b0110: DIN_CFG <= pattern_generation[6];
						4'b0111: DIN_CFG <= pattern_generation[7];
						4'b1000: DIN_CFG <= pattern_generation[8];
						4'b1001: DIN_CFG <= pattern_generation[9];
						4'b1010: DIN_CFG <= pattern_generation[10];
						4'b1011: DIN_CFG <= pattern_generation[11];
						4'b1100:
							begin
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[12];
								else DIN_CFG <= 1'b0;
							end
						4'b1101:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[13];
								else DIN_CFG <= 1'b0;
							end
						4'b1110:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[14];
								else DIN_CFG <= 1'b0;
							end
						4'b1111:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[15];
								else DIN_CFG <= 1'b0;
							end
						default: DIN_CFG <= 1'b0;
					endcase
				end
	
			CLK_CFG_HIGH_STATE:
				begin
					CLK_CFG <= 1'b1;
					if(count_high == 4'h0)
						begin
							FSMn <= CLK_CFG_LOW_STATE;
							load_counter_low <= 1'b1;
						end
					else
						begin
							dec_hig <= 1'b1;
							FSMn <= CLK_CFG_HIGH_STATE;
						end
					case(config_CLK[3:0])
						4'b0000: DIN_CFG <= pattern_generation[0];
						4'b0001: DIN_CFG <= pattern_generation[1];
						4'b0010: DIN_CFG <= pattern_generation[2];
						4'b0011: DIN_CFG <= pattern_generation[3];
						4'b0100: DIN_CFG <= pattern_generation[4];
						4'b0101: DIN_CFG <= pattern_generation[5];
						4'b0110: DIN_CFG <= pattern_generation[6];
						4'b0111: DIN_CFG <= pattern_generation[7];
						4'b1000: DIN_CFG <= pattern_generation[8];
						4'b1001: DIN_CFG <= pattern_generation[9];
						4'b1010: DIN_CFG <= pattern_generation[10];
						4'b1011: DIN_CFG <= pattern_generation[11];
						4'b1100:
							begin
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[12];
								else DIN_CFG <= 1'b0;
							end
						4'b1101:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[13];
								else DIN_CFG <= 1'b0;
							end
						4'b1110:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[14];
								else DIN_CFG <= 1'b0;
							end
						4'b1111:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[15];
								else DIN_CFG <= 1'b0;
							end
						default: DIN_CFG <= 1'b0;
					endcase
				end
	
			CLK_CFG_LOW_STATE:
				begin
					if(count_low == 4'h0)
						begin
							if(config_CLK == 4'h0)
								begin
									FSMn <= DELAY_WRITE_READ;
									load_counter_aux_read_write <= 1'b1;
								end
							else
								begin
									FSMn <= WAIT;
									dec_config <= 1'b1;
								end
						end
					else
						begin
							FSMn <= CLK_CFG_LOW_STATE;
							dec_low <= 1'b1;
						end
					case(config_CLK[3:0])
						4'b0000: DIN_CFG <= pattern_generation[0];
						4'b0001: DIN_CFG <= pattern_generation[1];
						4'b0010: DIN_CFG <= pattern_generation[2];
						4'b0011: DIN_CFG <= pattern_generation[3];
						4'b0100: DIN_CFG <= pattern_generation[4];
						4'b0101: DIN_CFG <= pattern_generation[5];
						4'b0110: DIN_CFG <= pattern_generation[6];
						4'b0111: DIN_CFG <= pattern_generation[7];
						4'b1000: DIN_CFG <= pattern_generation[8];
						4'b1001: DIN_CFG <= pattern_generation[9];
						4'b1010: DIN_CFG <= pattern_generation[10];
						4'b1011: DIN_CFG <= pattern_generation[11];
						4'b1100:
							begin
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[12];
								else DIN_CFG <= 1'b0;
							end
						4'b1101:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[13];
								else DIN_CFG <= 1'b0;
							end
						4'b1110:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[14];
								else DIN_CFG <= 1'b0;
							end
						4'b1111:
							begin 
								if(FORMAT == 1'b1) DIN_CFG <= pattern_generation[15];
								else DIN_CFG <= 1'b0;
							end
						default: DIN_CFG <= 1'b0;
					endcase
				end					
					
			DELAY_WRITE_READ:
				begin
					set_up <= 1'b1;
					if(COUNTER_AUX_READ_WRITE == 8'h00)
						begin
							FSMn <= SET_START_CONVERSION;
						end
					else
						begin
							FSMn <= DELAY_WRITE_READ;
							dec_counter_aux_red_write <= 1'b1;
						end
				end
	
			SET_START_CONVERSION:
				begin
					set_up <= 1'b1;
					start_conv <= 1'b1;
					load_counter_aux <= 1'b1;
					FSMn <= WAIT_1;
				end
	
			WAIT_1:
				begin
					start_conv <= 1'b1;
					if(COUNTER_AUX == 3'h0)
						begin
							FSMn <= SET_DVALID;
						end
					else
						begin
							FSMn <= WAIT_1;	
							dec_aux_count <= 1'b1;
						end
				end
	
			SET_DVALID:
				begin
					FSMn <= WAIT_FOR_DONE;
					start_conv <= 1'b1;
					dvalid_gen_control <= 1'b1;
				end
	
			DELAY:  //not actually used
				begin
					start_conv <= 1'b1;
					if(COUNTER_AUX == 3'h0)
						begin
							FSMn <= WAIT_FOR_DONE;
						end
					else
						begin
							FSMn <= DELAY;	
							dvalid_gen_control <= 1'b1;
							dec_aux_count <= 1'b1;
						end
				end
	
			WAIT_FOR_DONE:
				begin
					start_conv <= 1'b1;
					if(DONE_READ == 1'b1) 
						begin
							FSMn <= DONE_START_CONVERSION;
						end
					else
						begin
							FSMn <= WAIT_FOR_DONE;
						end
				end
	
			DONE_START_CONVERSION:
				begin
					FSMn <= DONE;
					start_conv <= 1'b0;
				end
	
			DONE:
				begin
					if(TRIGGER_i == 1'b1)
						begin
							FSMn <= DONE;
						end
					else
						begin
							FSMn <= IDLE;
						end
				end
				
		endcase
	end

endmodule


