`timescale 1ns / 100ps
module RAM_CONTROL(	FPGA_CLK,
							FPGA_RESET,

							DATA_FROM_CAPTURE_FIFO,
							DATA_FROM_RAM,

							RAM_ADDR_OUT,	  // the actual ram address on the port
							RAM_DATA,
							//PUT_DATA_ON_PORT,

							nCS1,
							nOE_PASSTHROUGH,
							nWE_PASSTHROUGH,

							// Communications with DDC_DATA_CAP
							RESET_ADDRESS_COUNTERS,

							// Communications with DDC_DATA_FIFO
							RESERVE_RAM_FOR_WRITING,
							DATA_FROM_DDC_FIFO_AVAILABLE,	// this is 1'b1 when the DDC_DATA_FIFO shows !empty
							RAM_WRITE_DATA_REQUESTED,  // this is how we say we want the data to be loaded onto DATA_FROM_CAPTURE_FIFO[15:0]
							RAM_WRITE_DATA_PROCESSED,
							//DDC_FIFO_WRITE_DATA_LOADED, // this goes to 1 when data is loaded onto DATA_FROM_CAPTURE_FIFO[15:0]
							//we set RAM_WRITE_DATA_REQUESTED low when we are done writing to the ram
							READY_TO_WRITE,

							// Communications with RAM_TO_USB_FIFO
							DATA_STORED_IN_RAM,
						   REQUEST_RAM_DATA,
							DATA_AVAILABLE_FROM_RAM,
							//DATA_FROM_RAM_PROCESSED
							//the other module sets REQUEST_RAM_DATA low when it has processed the data
							USE_RAM_CHIPS
							);

//Port Declarations
input FPGA_CLK;
input FPGA_RESET;
input [15:0] DATA_FROM_CAPTURE_FIFO;

output [15:0] DATA_FROM_RAM;
wire [15:0] DATA_FROM_RAM;

output [19:0] RAM_ADDR_OUT;
wire [19:0] RAM_ADDR_OUT;

inout [15:0] RAM_DATA;
wire [15:0] RAM_DATA;

//output PUT_DATA_ON_PORT;
//reg PUT_DATA_ON_PORT;

output nCS1, nOE_PASSTHROUGH, nWE_PASSTHROUGH;
wire nCS1, nOE_PASSTHROUGH, nWE_PASSTHROUGH;

input RESET_ADDRESS_COUNTERS;
input RESERVE_RAM_FOR_WRITING;

input DATA_FROM_DDC_FIFO_AVAILABLE;

output RAM_WRITE_DATA_REQUESTED;
reg RAM_WRITE_DATA_REQUESTED;

//input DDC_FIFO_WRITE_DATA_LOADED;

output RAM_WRITE_DATA_PROCESSED;
reg RAM_WRITE_DATA_PROCESSED;

input READY_TO_WRITE;

output DATA_STORED_IN_RAM;
wire DATA_STORED_IN_RAM;

input REQUEST_RAM_DATA;
reg request_ram_data_gated;

output DATA_AVAILABLE_FROM_RAM;
reg DATA_AVAILABLE_FROM_RAM; 


input USE_RAM_CHIPS;

//input DATA_FROM_RAM_PROCESSED;


// Local Variable Declarations

reg [7:0] FSM, FSMn;

wire [19:0] ram_addr_counter;

reg [19:0] ram_fifo_addr_out;  //the address of the data output to the USB FIFO
reg reset_ram_fifo_addr_out_i; //always less than or equal to ram_fifo_addr_in
reg incr_ram_fifo_addr_out_i;
									  
reg [19:0] ram_fifo_addr_in;	 //the address of the next RAM location to be stored
reg reset_ram_fifo_addr_in_i;	 //always greater than or equal to ram_fifo_addr_out
reg incr_ram_fifo_addr_in_i;

reg r_write_n, r_write_n_i;
reg r_read_n, r_read_n_i;

wire nCS_output_w;
wire w_ram_wait;

reg ram_write_data_requested_i;

reg data_available_from_ram_i;

reg ram_write_data_processed_i;

//reg put_data_on_port_i;




// Local Parameter Declarations

parameter 	IDLE_1			= 8'b00000000,
				LATCH_IN_2		= 8'b00000001,
				LATCH_IN_3		= 8'b00000010,
				WAIT_4			= 8'b00000011,
				FETCH_DATA_5	= 8'b00000100,
				WAIT_6			= 8'b00000101,
				WAIT_4_1			= 8'b00000110,
				WAIT_6_1			= 8'b00001000;

parameter	IDLE = 8'h00,
				BEGIN_RAM_WRITE_STATE = 8'h01,
				DELAY_RAM_WRITE_STATE = 8'h02,
				DELAY_RAM_WRITE_STATE_2 = 8'h03,
				FINALIZE_WRITE_STATE = 8'h0F,
				BEGIN_RAM_READ_STATE = 8'h11,
				DELAY_RAM_READ_STATE = 8'h12,
				FINALIZE_READ_STATE = 8'h1F,
				WAIT_READ_STATE1 = 8'h1E,
				START_READ_STATE = 8'h1D,
				BEGIN_RAM_READ_STATE2 = 8'h1C,
				DELAY_RAM_READ_STATE2 = 8'h1B;





// Assignments

assign ram_addr_counter[19:0] = (RESERVE_RAM_FOR_WRITING==1'b1) ? ram_fifo_addr_in[19:0] : ram_fifo_addr_out[19:0];
	// RESERVE_RAM_FOR_WRITING means that the part is running and is reserving the right to write to the RAM.

assign RAM_ADDR_OUT[19:0] = ram_addr_counter[19:0];

assign nCS1 = nCS_output_w;

assign DATA_STORED_IN_RAM = (ram_fifo_addr_in[19:0]==ram_fifo_addr_out[19:0]) ? 1'b0 : 1'b1;
//assign DATA_FROM_RAM[15:0] = ram_fifo_addr_out[15:0];

//Module Declarations

MEMORY memory(	.CLK(FPGA_CLK), 
					.RESET(FPGA_RESET), 
					.WR_N(r_write_n), 
					.RD_N(r_read_n), 
					.DIN(DATA_FROM_CAPTURE_FIFO[15:0]),  //part passthrough
					//.DIN(ram_fifo_addr_in[15:0]),
					.DOUT(DATA_FROM_RAM),  //passthrough
					.RAM_DATA(RAM_DATA), //passthrough
					.RAM_CS_N(nCS_output_w),  // goes to one pair of RAM chips or the other based on r_ram_addr_counter[19].
					.RAM_OE_N(nOE_PASSTHROUGH),  //passthrough
					.RAM_WE_N(nWE_PASSTHROUGH),  //passthrough
					.RAM_WAIT(w_ram_wait)
					);





// "Always @ CLK" blocks

always @(posedge FPGA_CLK or negedge FPGA_RESET)
  begin
    if(FPGA_RESET == 1'b0)
      begin
		   FSM <= IDLE;
		   ram_fifo_addr_out[19:0] <= 20'b0000_00000000_00000000;
		   ram_fifo_addr_in[19:0] <= 20'b0000_00000000_00000000;
		   r_write_n <= 1'b1;
		   r_read_n <= 1'b1;
			RAM_WRITE_DATA_REQUESTED <= 1'b0;
			DATA_AVAILABLE_FROM_RAM <= 1'b0;
//		   PUT_DATA_ON_PORT <= 1'b0;
         request_ram_data_gated<=1'b0;
         RAM_WRITE_DATA_PROCESSED<=1'b0;
		end
    else
      begin
			FSM <= FSMn;
			if(USE_RAM_CHIPS==1'b1)
				begin
					if(reset_ram_fifo_addr_out_i == 1'b1) ram_fifo_addr_out[19:0] <= 20'b0000_00000000_00000000;
					if(incr_ram_fifo_addr_out_i == 1'b1)  ram_fifo_addr_out[19:0] <= ram_fifo_addr_out[19:0] + 20'b0000_00000000_00000001; 
					if(reset_ram_fifo_addr_in_i == 1'b1) ram_fifo_addr_in[19:0] <= 20'b0000_00000000_00000000;
					if(incr_ram_fifo_addr_in_i == 1'b1)  ram_fifo_addr_in[19:0] <= ram_fifo_addr_in[19:0] + 20'b0000_00000000_00000001; 
				end
		   r_write_n <= r_write_n_i;
		   r_read_n <= r_read_n_i;
			RAM_WRITE_DATA_REQUESTED <= ram_write_data_requested_i;
			DATA_AVAILABLE_FROM_RAM <= data_available_from_ram_i;
//		   PUT_DATA_ON_PORT <= put_data_on_port_i;
         request_ram_data_gated<=REQUEST_RAM_DATA;
         RAM_WRITE_DATA_PROCESSED <= ram_write_data_processed_i;
		end
  end


// State Machines, combinational logic

always @(	FSM or
				RESERVE_RAM_FOR_WRITING or
				DATA_FROM_DDC_FIFO_AVAILABLE or
				//DDC_FIFO_WRITE_DATA_LOADED or
				READY_TO_WRITE or
				request_ram_data_gated or
				RESET_ADDRESS_COUNTERS or
				w_ram_wait 
				)
	begin
		FSMn<=FSM;
		reset_ram_fifo_addr_out_i<=1'b0;
		incr_ram_fifo_addr_out_i<=1'b0;
		reset_ram_fifo_addr_in_i<=1'b0;
		incr_ram_fifo_addr_in_i<=1'b0;
		r_write_n_i<=1'b1;  //reverse polarity
		r_read_n_i<=1'b1;  //reverse polarity
		ram_write_data_requested_i<=1'b0;
		data_available_from_ram_i<=1'b0;
//		put_data_on_port_i=1'b0;
      ram_write_data_processed_i<=1'b0;

		case(FSM)
			IDLE:
				begin
					if(RESET_ADDRESS_COUNTERS)
						begin
							FSMn<=IDLE;
							reset_ram_fifo_addr_out_i<=1'b1;
							reset_ram_fifo_addr_in_i<=1'b1;
						end
					else
						begin
							if(RESERVE_RAM_FOR_WRITING==1'b1 && DATA_FROM_DDC_FIFO_AVAILABLE==1'b1)
								begin
									FSMn<=BEGIN_RAM_WRITE_STATE;
//									put_data_on_port_i<=1'b1;
									ram_write_data_requested_i<=1'b1;
									r_write_n_i<=1'b0;
								end
							else
								begin
									if(RESERVE_RAM_FOR_WRITING==1'b0 && request_ram_data_gated==1'b1)
										begin
											FSMn<=BEGIN_RAM_READ_STATE;
											r_read_n_i<=1'b0;
										end
									else							  
										begin
											FSMn<=IDLE;
										end
								end
						end
				end
				
			BEGIN_RAM_WRITE_STATE:
			   begin
			      FSMn<=DELAY_RAM_WRITE_STATE;
//					put_data_on_port_i<=1'b1;
			      //ram_write_data_requested_i=1'b1;    ///trial to remove x3 write cycles
					r_write_n_i<=1'b0;
			   end
			   
			DELAY_RAM_WRITE_STATE:
			   begin
//				   put_data_on_port_i<=1'b1;
			      FSMn<=FINALIZE_WRITE_STATE;
			      //ram_write_data_requested_i=1'b1;
					//r_write_n_i=1'b0;
					ram_write_data_processed_i<=1'b1;
			   end
			   
			FINALIZE_WRITE_STATE:
				begin
					//if(w_ram_wait == 1'b1)
					//if(DATA_FROM_DDC_FIFO_AVAILABLE==1'b1)
					if(READY_TO_WRITE==1'b0)
						begin
							FSMn<=FINALIZE_WRITE_STATE;
							ram_write_data_processed_i<=1'b1;
							//ram_write_data_requested_i<=1'b1;
						end
				   else
						begin
							FSMn<=IDLE;
							incr_ram_fifo_addr_in_i<=1'b1;  //increment to next address for writing.
						end
				end




//			BEGIN_RAM_READ_STATE:
//				begin
//					FSMn=DELAY_RAM_READ_STATE;
//					r_read_n_i=1'b0;
//				end
//
//			DELAY_RAM_READ_STATE:
//				begin
//					FSMn=FINALIZE_READ_STATE;
//					//r_read_n_i=1'b0;
//				end


			BEGIN_RAM_READ_STATE:
				begin
					FSMn<=DELAY_RAM_READ_STATE;
					r_read_n_i<=1'b0;
				end

			DELAY_RAM_READ_STATE:
				begin
					FSMn<=WAIT_READ_STATE1;
				end

			WAIT_READ_STATE1:
				begin
					if(w_ram_wait==1'b1) FSMn<=WAIT_READ_STATE1;
					else FSMn<=START_READ_STATE;
				end

			START_READ_STATE:
				begin
					FSMn<=BEGIN_RAM_READ_STATE2;
					r_read_n_i<=1'b0;
				end
			
			BEGIN_RAM_READ_STATE2:
				begin
					FSMn<=DELAY_RAM_READ_STATE2;
					r_read_n_i<=1'b0;
				end

			DELAY_RAM_READ_STATE2:
				begin
					FSMn<=FINALIZE_READ_STATE;
				end

			FINALIZE_READ_STATE:
				begin
					data_available_from_ram_i<=1'b1;
					if(w_ram_wait==1'b1 || request_ram_data_gated==1'b1)  //when REQUEST_RAM_DATA goes to 0, then the data has been processed.
						begin
							FSMn<=FINALIZE_READ_STATE;
						end
					else
						begin
							FSMn<=IDLE;
							incr_ram_fifo_addr_out_i<=1'b1;
						end
				end




			default:
				begin
					FSMn<=IDLE;
				end
		endcase

	end //always











endmodule
