`timescale 1ns / 100ps
//`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:    14:48:33 03/28/06
// Design Name:    
// Module Name:    RAM_TO_USB_FIFO
// Project Name:   
// Target Device:  
// Tool versions:  
// Description:
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////
module RAM_TO_USB_FIFO(	FPGA_CLK,
								FPGA_RESET,

								//RAM_CONTROL interface
								DATA_FROM_RAM, //16 bits
								
								RAM_RESERVED_FOR_WRITING, //this is set when the device is running, storing data to the RAM

								DATA_STORED_IN_RAM, //this is high when ram address in != ram address out
								REQUEST_RAM_DATA, // The FIFO sets this when sr_full=0 and when data_stored_in_RAM=1
								DATA_AVAILABLE_FROM_RAM, //DDC_DATA_CAPTURE sets this to 1 when new data is available for latching
								//DATA_FROM_RAM_PROCESSED, //we set this when we're done parsing and loading ddc data into the fifo

								//USB_IO interface
								DATA_FROM_FIFO_TO_USB, //always 16 bits.
								
								RAM_FIFO_WRITE_DATA_AVAILABLE, //we set this when we have data for the USB_IO block to send back
								USB_WRITE_DATA_REQUESTED,  //USB_IO sets this to 1 when it is ready for our data
								RAM_FIFO_WRITE_DATA_LOADED,  //we set this when the data for USB_IO is on the DATA_FROM_FIFO_TO_USB lines
								USB_WRITE_DATA_PROCESSED,  //USB_IO sets this to 1 when it is done with the data on the lines.  we can advance...

								PKTEND_INHIBITOR,
								PKTEND_SEND,
								PKTEND_SENT

								//latch_out,
								//fifo_address_out
								);

//////////// DECLARATIONS ////////////
input FPGA_CLK, FPGA_RESET; 

input [15:0] DATA_FROM_RAM;

input RAM_RESERVED_FOR_WRITING;

input DATA_STORED_IN_RAM;
reg data_stored_in_ram_gated;

output REQUEST_RAM_DATA;
reg REQUEST_RAM_DATA;

input DATA_AVAILABLE_FROM_RAM;
reg data_available_from_ram_gated;

//output DATA_FROM_RAM_PROCESSED;
//reg DATA_FROM_RAM_PROCESSED;

output [15:0] DATA_FROM_FIFO_TO_USB;
wire [15:0] DATA_FROM_FIFO_TO_USB;

output RAM_FIFO_WRITE_DATA_AVAILABLE;
//wire RAM_FIFO_WRITE_DATA_AVAILABLE;
reg RAM_FIFO_WRITE_DATA_AVAILABLE;

input USB_WRITE_DATA_REQUESTED;

output RAM_FIFO_WRITE_DATA_LOADED;
reg RAM_FIFO_WRITE_DATA_LOADED;

input USB_WRITE_DATA_PROCESSED;

output PKTEND_INHIBITOR,
       PKTEND_SEND;
reg    PKTEND_INHIBITOR,
       PKTEND_SEND;
       
input PKTEND_SENT;

//output latch_out;
//wire latch_out;


////////////////////// LOCAL VARIABLE DECLARATIONS ///////////////////////

reg request_ram_data_i;
//reg data_from_ram_processed_i;
//reg incr_ram_out_address_i;
reg ram_reserved_for_writing_gated;  //to sync with clock.
reg usb_write_data_requested_gated;  //to sync with clock.
reg ram_fifo_write_data_loaded_i;
reg usb_write_data_processed_gated; //to sync with clock.

wire pktend_inhibitor_i;
wire pktend_send_i;
//reg pktend_sent_gated;

reg RAM_FIFO_WRITE_DATA_AVAILABLE_i;

//wire sr_full;

wire sr_empty;
//reg sr_empty_gated;

reg increment_fpga_fifo_out_addr_i;
reg [3:0] fifo_address_in;
//output [3:0] fifo_address_out;
reg [3:0] fifo_address_out;


//reg [2:0] FSM, FSMn;
reg [3:0] FSM2, FSM2n;

reg [15:0] fpga_fifo_full_empty;  //0 indicates empty, 1 indicates full.  all-or indicates all empty, all-and indicates all full
wire [15:0] fpga_fifo;


//////////// PARAMETERS ////////////



parameter 	IDLE_2=4'h0,
				PRE_REQUEST_RAM_FETCH_2=4'h1,
				REQUEST_RAM_FETCH_2=4'h2,
				STORE_DATA_2=4'h3,
				PRE_WAIT_FOR_ACK_2=4'h4,
				WAIT_FOR_ACK_2=4'h5,
				WAIT_ONE_MORE_2=4'h6,

				LATCHED_DATA_NOW_WAIT_FOR_ACK = 4'hA,
				WAIT_1 = 4'hB,
				WAIT_2 = 4'hC,
				WAIT_3 = 4'hD;



//////////// ASSIGNMENTS ////////////
assign sr_empty=~(|fpga_fifo_full_empty[15:0]);
//assign sr_full=&fpga_fifo_full_empty[15:0];
//assign RAM_FIFO_WRITE_DATA_AVAILABLE = !sr_empty;

assign pktend_inhibitor_i=!sr_empty;
assign pktend_send_i=1'b0;




//////////// FPGA_FIFO SHIFT REGISTER ////////////


reg [15:0] fpga_fifo_0000;
reg [15:0] fpga_fifo_0001;
reg [15:0] fpga_fifo_0010;
reg [15:0] fpga_fifo_0011;
reg [15:0] fpga_fifo_0100;
reg [15:0] fpga_fifo_0101;
reg [15:0] fpga_fifo_0110;
reg [15:0] fpga_fifo_0111;
reg [15:0] fpga_fifo_1000;
reg [15:0] fpga_fifo_1001;
reg [15:0] fpga_fifo_1010;
reg [15:0] fpga_fifo_1011;
reg [15:0] fpga_fifo_1100;
reg [15:0] fpga_fifo_1101;
reg [15:0] fpga_fifo_1110;
reg [15:0] fpga_fifo_1111;
reg latch_data16_0_i;
reg latch_data16_1_i;
reg latch_data16_2_i;
reg latch_data16_3_i;
reg latch_data16_4_i;
reg latch_data16_5_i;
reg latch_data16_6_i;
reg latch_data16_7_i;
reg latch_data16_8_i;
reg latch_data16_9_i;
reg latch_data16_A_i;
reg latch_data16_B_i;
reg latch_data16_C_i;
reg latch_data16_D_i;
reg latch_data16_E_i;
reg latch_data16_F_i;

//assign latch_out = latch_data16_0_i || latch_data16_1_i || latch_data16_2_i || latch_data16_3_i || latch_data16_4_i || latch_data16_5_i || latch_data16_6_i || latch_data16_7_i || latch_data16_8_i || latch_data16_9_i || latch_data16_A_i || latch_data16_B_i || latch_data16_C_i || latch_data16_D_i || latch_data16_E_i || latch_data16_F_i;
//assign latch_out = increment_fpga_fifo_out_addr_i;
//assign latch_out = (FSM==LATCHED_DATA_NOW_WAIT_FOR_ACK)?1'b1:1'b0;

assign fpga_fifo[15:0] = (fifo_address_out[3:0]==4'b0000)?fpga_fifo_0000[15:0]: (fifo_address_out[3:0]==4'b0001)?fpga_fifo_0001[15:0]: (fifo_address_out[3:0]==4'b0010)?fpga_fifo_0010[15:0]: (fifo_address_out[3:0]==4'b0011)?fpga_fifo_0011[15:0]: (fifo_address_out[3:0]==4'b0100)?fpga_fifo_0100[15:0]: (fifo_address_out[3:0]==4'b0101)?fpga_fifo_0101[15:0]: (fifo_address_out[3:0]==4'b0110)?fpga_fifo_0110[15:0]: (fifo_address_out[3:0]==4'b0111)?fpga_fifo_0111[15:0]: (fifo_address_out[3:0]==4'b1000)?fpga_fifo_1000[15:0]: (fifo_address_out[3:0]==4'b1001)?fpga_fifo_1001[15:0]: (fifo_address_out[3:0]==4'b1010)?fpga_fifo_1010[15:0]: (fifo_address_out[3:0]==4'b1011)?fpga_fifo_1011[15:0]: (fifo_address_out[3:0]==4'b1100)?fpga_fifo_1100[15:0]: (fifo_address_out[3:0]==4'b1101)?fpga_fifo_1101[15:0]: (fifo_address_out[3:0]==4'b1110)?fpga_fifo_1110[15:0]: /*fifo_address_in[3:0]==4'b1111?*/ fpga_fifo_1111[15:0];
assign DATA_FROM_FIFO_TO_USB[15:0] = fpga_fifo[15:0];

always @(posedge FPGA_CLK or negedge FPGA_RESET)
  begin
  if(FPGA_RESET==1'b0)
    begin
	 fpga_fifo_0000[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_0001[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_0010[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_0011[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_0100[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_0101[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_0110[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_0111[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1000[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1001[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1010[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1011[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1100[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1101[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1110[15:0]=16'b0000_0000__0000_0000;
	 fpga_fifo_1111[15:0]=16'b0000_0000__0000_0000;
	 fifo_address_in[3:0]=4'b0000;
	 fpga_fifo_full_empty[15:0]=16'b0000_0000__0000_0000;
    end
  else
	 begin
	 	if(increment_fpga_fifo_out_addr_i==1'b1)
		   begin
			if(fifo_address_out[3:0]==4'b0000) fpga_fifo_full_empty[0]=1'b0;
			if(fifo_address_out[3:0]==4'b0001) fpga_fifo_full_empty[1]=1'b0;
			if(fifo_address_out[3:0]==4'b0010) fpga_fifo_full_empty[2]=1'b0;
			if(fifo_address_out[3:0]==4'b0011) fpga_fifo_full_empty[3]=1'b0;
			if(fifo_address_out[3:0]==4'b0100) fpga_fifo_full_empty[4]=1'b0;
			if(fifo_address_out[3:0]==4'b0101) fpga_fifo_full_empty[5]=1'b0;
			if(fifo_address_out[3:0]==4'b0110) fpga_fifo_full_empty[6]=1'b0;
			if(fifo_address_out[3:0]==4'b0111) fpga_fifo_full_empty[7]=1'b0;
			if(fifo_address_out[3:0]==4'b1000) fpga_fifo_full_empty[8]=1'b0;
			if(fifo_address_out[3:0]==4'b1001) fpga_fifo_full_empty[9]=1'b0;
			if(fifo_address_out[3:0]==4'b1010) fpga_fifo_full_empty[10]=1'b0;
			if(fifo_address_out[3:0]==4'b1011) fpga_fifo_full_empty[11]=1'b0;
			if(fifo_address_out[3:0]==4'b1100) fpga_fifo_full_empty[12]=1'b0;
			if(fifo_address_out[3:0]==4'b1101) fpga_fifo_full_empty[13]=1'b0;
			if(fifo_address_out[3:0]==4'b1110) fpga_fifo_full_empty[14]=1'b0;
			if(fifo_address_out[3:0]==4'b1111) fpga_fifo_full_empty[15]=1'b0;
			end
		else
			begin
				if(latch_data16_0_i==1'b1)
					begin
					fpga_fifo_0000[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[0]=1'b1;
					fifo_address_in[3:0]=4'h1;
					end
					
				if(latch_data16_1_i==1'b1)
					begin
					fpga_fifo_0001[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[1]=1'b1;
					fifo_address_in[3:0]=4'h2;
					end
					
				if(latch_data16_2_i==1'b1)
					begin
					fpga_fifo_0010[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[2]=1'b1;
					fifo_address_in[3:0]=4'h3;
					end
					
				if(latch_data16_3_i==1'b1)
					begin
					fpga_fifo_0011[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[3]=1'b1;
					fifo_address_in[3:0]=4'h4;
					end
					
				if(latch_data16_4_i==1'b1)
					begin
					fpga_fifo_0100[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[4]=1'b1;
					fifo_address_in[3:0]=4'h5;
					end
					
				if(latch_data16_5_i==1'b1)
					begin
					fpga_fifo_0101[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[5]=1'b1;
					fifo_address_in[3:0]=4'h6;
					end
					
				if(latch_data16_6_i==1'b1)
					begin
					fpga_fifo_0110[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[6]=1'b1;
					fifo_address_in[3:0]=4'h7;
					end
					
				if(latch_data16_7_i==1'b1)
					begin
					fpga_fifo_0111[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[7]=1'b1;
					fifo_address_in[3:0]=4'h8;
					end
					
				if(latch_data16_8_i==1'b1)
					begin
					fpga_fifo_1000[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[8]=1'b1;
					fifo_address_in[3:0]=4'h9;
					end
					
				if(latch_data16_9_i==1'b1)
					begin
					fpga_fifo_1001[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[9]=1'b1;
					fifo_address_in[3:0]=4'hA;
					end
					
				if(latch_data16_A_i==1'b1)
					begin
					fpga_fifo_1010[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[10]=1'b1;
					fifo_address_in[3:0]=4'hB;
					end
					
				if(latch_data16_B_i==1'b1)
					begin
					fpga_fifo_1011[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[11]=1'b1;
					fifo_address_in[3:0]=4'hC;
					end
					
				if(latch_data16_C_i==1'b1)
					begin
					fpga_fifo_1100[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[12]=1'b1;
					fifo_address_in[3:0]=4'hD;
					end
					
				if(latch_data16_D_i==1'b1)
					begin
					fpga_fifo_1101[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[13]=1'b1;
					fifo_address_in[3:0]=4'hE;
					end
					
				if(latch_data16_E_i==1'b1)
					begin
					fpga_fifo_1110[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[14]=1'b1;
					fifo_address_in[3:0]=4'hF;
					end
					
				if(latch_data16_F_i==1'b1)
					begin
					fpga_fifo_1111[15:0]=DATA_FROM_RAM[15:0];
					fpga_fifo_full_empty[15]=1'b1;
					fifo_address_in[3:0]=4'h0;
					end
			end
	 end
  end




//////////// ALWAYS BLOCKS ////////////

always @(posedge FPGA_CLK or negedge FPGA_RESET)
  begin
  if(FPGA_RESET==1'b0)
    begin
    //FSM=3'b000;
	 //FSMn=3'b111;
	 FSM2=IDLE_2;	  
	 REQUEST_RAM_DATA=1'b0;
    RAM_FIFO_WRITE_DATA_LOADED=1'b0;
    //DATA_FROM_RAM_PROCESSED=1'b0;
    usb_write_data_processed_gated=1'b0;
    usb_write_data_requested_gated=1'b0;
    ram_reserved_for_writing_gated =1'b0;
	 data_stored_in_ram_gated=1'b0;
	 data_available_from_ram_gated=1'b0;
    fifo_address_out[3:0] = 4'h0;
	 PKTEND_INHIBITOR=1'b1;
	 PKTEND_SEND=1'b0;
	 //pktend_sent_gated=1'b0;
	 RAM_FIFO_WRITE_DATA_AVAILABLE=1'b0;
	 //sr_empty_gated=1'b1;
    end
  else
    begin
	    //FSM=FSMn;
		 FSM2=FSM2n;
		 REQUEST_RAM_DATA=request_ram_data_i;
	    RAM_FIFO_WRITE_DATA_LOADED=ram_fifo_write_data_loaded_i;
		 //DATA_FROM_RAM_PROCESSED=data_from_ram_processed_i;
	    usb_write_data_processed_gated=USB_WRITE_DATA_PROCESSED;
	    usb_write_data_requested_gated=USB_WRITE_DATA_REQUESTED;
	    ram_reserved_for_writing_gated=RAM_RESERVED_FOR_WRITING;
		 data_stored_in_ram_gated = DATA_STORED_IN_RAM;
		 data_available_from_ram_gated = DATA_AVAILABLE_FROM_RAM;
		 PKTEND_INHIBITOR=pktend_inhibitor_i;
		 PKTEND_SEND=pktend_send_i;
		 //pktend_sent_gated=PKTEND_SENT;
		 RAM_FIFO_WRITE_DATA_AVAILABLE=RAM_FIFO_WRITE_DATA_AVAILABLE_i;
		 //sr_empty_gated=sr_empty;

	    case( {increment_fpga_fifo_out_addr_i,fifo_address_out[3:0]} )
		 	5'h10:
				begin
		 			fifo_address_out[3:0] = 4'h1;
				end
		 	5'h11:
				begin
		 			fifo_address_out[3:0] = 4'h2;
				end
		 	5'h12:
				begin
		 			fifo_address_out[3:0] = 4'h3;
				end
		 	5'h13:
				begin
		 			fifo_address_out[3:0] = 4'h4;
				end
		 	5'h14:
				begin
		 			fifo_address_out[3:0] = 4'h5;
				end
		 	5'h15:
				begin
		 			fifo_address_out[3:0] = 4'h6;
				end
		 	5'h16:
				begin
		 			fifo_address_out[3:0] = 4'h7;
				end
		 	5'h17:
				begin
		 			fifo_address_out[3:0] = 4'h8;
				end
		 	5'h18:
				begin
		 			fifo_address_out[3:0] = 4'h9;
				end
		 	5'h19:
				begin
		 			fifo_address_out[3:0] = 4'hA;
				end
		 	5'h1A:
				begin
		 			fifo_address_out[3:0] = 4'hB;
				end
		 	5'h1B:
				begin
		 			fifo_address_out[3:0] = 4'hC;
				end
		 	5'h1C:
				begin
		 			fifo_address_out[3:0] = 4'hD;
				end
		 	5'h1D:
				begin
		 			fifo_address_out[3:0] = 4'hE;
				end
		 	5'h1E:
				begin
		 			fifo_address_out[3:0] = 4'hF;
				end
		 	5'h1F:
				begin
		 			fifo_address_out[3:0] = 4'h0;
				end
		 	default:
				begin
		 			fifo_address_out[3:0] = fifo_address_out[3:0];
				end
		endcase
    end
  end








//////////// STATE MACHINES ////////////

// State Machine to talk to USB_IO



// State Machine to talk to RAM_CONTROL   and USB_IO
always @(	FSM2 or
				ram_reserved_for_writing_gated or
				data_stored_in_ram_gated or
				data_available_from_ram_gated or
				fifo_address_in or
				fpga_fifo_full_empty or
				usb_write_data_requested_gated or
				usb_write_data_processed_gated or
				sr_empty or
				RAM_FIFO_WRITE_DATA_AVAILABLE
				)
	begin
		FSM2n=FSM2;
		latch_data16_0_i=1'b0;
		latch_data16_1_i=1'b0;
		latch_data16_2_i=1'b0;
		latch_data16_3_i=1'b0;
		latch_data16_4_i=1'b0;
		latch_data16_5_i=1'b0;
		latch_data16_6_i=1'b0;
		latch_data16_7_i=1'b0;
		latch_data16_8_i=1'b0;
		latch_data16_9_i=1'b0;
		latch_data16_A_i=1'b0;
		latch_data16_B_i=1'b0;
		latch_data16_C_i=1'b0;
		latch_data16_D_i=1'b0;
		latch_data16_E_i=1'b0;
		latch_data16_F_i=1'b0;
		request_ram_data_i=1'b0;
		//data_from_ram_processed_i=1'b0;
	   ram_fifo_write_data_loaded_i=1'b0;
	   increment_fpga_fifo_out_addr_i=1'b0;
	   RAM_FIFO_WRITE_DATA_AVAILABLE_i=1'b0;

		case(FSM2)
			IDLE_2:
				begin
					if( (ram_reserved_for_writing_gated==1'b0 && data_stored_in_ram_gated==1'b1) && ( (fifo_address_in[3:0]==4'h0 && fpga_fifo_full_empty[0]==1'b0) || (fifo_address_in[3:0]==4'h1 && fpga_fifo_full_empty[1]==1'b0) || (fifo_address_in[3:0]==4'h2 && fpga_fifo_full_empty[2]==1'b0) || (fifo_address_in[3:0]==4'h3 && fpga_fifo_full_empty[3]==1'b0) || (fifo_address_in[3:0]==4'h4 && fpga_fifo_full_empty[4]==1'b0) || (fifo_address_in[3:0]==4'h5 && fpga_fifo_full_empty[5]==1'b0) || (fifo_address_in[3:0]==4'h6 && fpga_fifo_full_empty[6]==1'b0) || (fifo_address_in[3:0]==4'h7 && fpga_fifo_full_empty[7]==1'b0) || (fifo_address_in[3:0]==4'h8 && fpga_fifo_full_empty[8]==1'b0) || (fifo_address_in[3:0]==4'h9 && fpga_fifo_full_empty[9]==1'b0) || (fifo_address_in[3:0]==4'hA && fpga_fifo_full_empty[10]==1'b0) || (fifo_address_in[3:0]==4'hB && fpga_fifo_full_empty[11]==1'b0) || (fifo_address_in[3:0]==4'hC && fpga_fifo_full_empty[12]==1'b0) || (fifo_address_in[3:0]==4'hD && fpga_fifo_full_empty[13]==1'b0) || (fifo_address_in[3:0]==4'hE && fpga_fifo_full_empty[14]==1'b0) || (fifo_address_in[3:0]==4'hF && fpga_fifo_full_empty[15]==1'b0) ) )
						begin
							FSM2n=PRE_REQUEST_RAM_FETCH_2;
							//request_ram_data_i=1'b1;
						end
					else
						begin
						   if(usb_write_data_requested_gated==1'b1 && RAM_FIFO_WRITE_DATA_AVAILABLE) //data is requested from USB_IO, shift reg not empty.
						     begin
							     FSM2n=LATCHED_DATA_NOW_WAIT_FOR_ACK;
						     end
							else
							  begin
							    FSM2n=IDLE_2;
					 		    if(sr_empty==1'b0)
							      begin
								     RAM_FIFO_WRITE_DATA_AVAILABLE_i=1'b1;
								   end
							  end
						end
				end







    LATCHED_DATA_NOW_WAIT_FOR_ACK:
      begin
	   if(usb_write_data_processed_gated==1'b0) //still processing data on USB_IO side
	     begin
	       FSM2n=LATCHED_DATA_NOW_WAIT_FOR_ACK;
			 ram_fifo_write_data_loaded_i=1'b1;
	     end
	   else //INCREMENT_FGPA_FIFO_OUT_ADDRESS_AFTER_WRITE
	     begin
		    FSM2n=WAIT_1;
			 ram_fifo_write_data_loaded_i=1'b1;
		    increment_fpga_fifo_out_addr_i=1'b1;
	     end
	   end

    WAIT_1:
	   begin
		   FSM2n=WAIT_2;
		end

    WAIT_2:
	   begin
		   FSM2n=WAIT_3;
		end

    WAIT_3:
	   begin
		   FSM2n=IDLE_2;
		end







			PRE_REQUEST_RAM_FETCH_2:
				begin
					if( (ram_reserved_for_writing_gated==1'b0 && data_stored_in_ram_gated==1'b1) && ( (fifo_address_in[3:0]==4'h0 && fpga_fifo_full_empty[0]==1'b0) || (fifo_address_in[3:0]==4'h1 && fpga_fifo_full_empty[1]==1'b0) || (fifo_address_in[3:0]==4'h2 && fpga_fifo_full_empty[2]==1'b0) || (fifo_address_in[3:0]==4'h3 && fpga_fifo_full_empty[3]==1'b0) || (fifo_address_in[3:0]==4'h4 && fpga_fifo_full_empty[4]==1'b0) || (fifo_address_in[3:0]==4'h5 && fpga_fifo_full_empty[5]==1'b0) || (fifo_address_in[3:0]==4'h6 && fpga_fifo_full_empty[6]==1'b0) || (fifo_address_in[3:0]==4'h7 && fpga_fifo_full_empty[7]==1'b0) || (fifo_address_in[3:0]==4'h8 && fpga_fifo_full_empty[8]==1'b0) || (fifo_address_in[3:0]==4'h9 && fpga_fifo_full_empty[9]==1'b0) || (fifo_address_in[3:0]==4'hA && fpga_fifo_full_empty[10]==1'b0) || (fifo_address_in[3:0]==4'hB && fpga_fifo_full_empty[11]==1'b0) || (fifo_address_in[3:0]==4'hC && fpga_fifo_full_empty[12]==1'b0) || (fifo_address_in[3:0]==4'hD && fpga_fifo_full_empty[13]==1'b0) || (fifo_address_in[3:0]==4'hE && fpga_fifo_full_empty[14]==1'b0) || (fifo_address_in[3:0]==4'hF && fpga_fifo_full_empty[15]==1'b0) ) )
						begin
							FSM2n=REQUEST_RAM_FETCH_2;
							request_ram_data_i=1'b1;
						end
					else
						begin
							FSM2n=IDLE_2;
						end
				end

			REQUEST_RAM_FETCH_2:
				begin			
					if(data_available_from_ram_gated==1'b1)
						begin
							FSM2n=STORE_DATA_2;
							request_ram_data_i=1'b1;
						end
					else
						begin
							FSM2n=REQUEST_RAM_FETCH_2;
							request_ram_data_i=1'b1;
						end
				end
	
			STORE_DATA_2:
			begin		
				FSM2n=PRE_WAIT_FOR_ACK_2;
				//data_from_ram_processed_i=1'b1;
				request_ram_data_i=1'b1;
				case(fifo_address_in[3:0])
					4'b0000:
					begin
						latch_data16_0_i=1'b1;
					end
					4'b0001:
					begin
						latch_data16_1_i=1'b1;
					end
					4'b0010:
					begin
						latch_data16_2_i=1'b1;
					end  
					4'b0011:
					begin
						latch_data16_3_i=1'b1;
					end
					4'b0100:
					begin
						latch_data16_4_i=1'b1;
					end
					4'b0101:
					begin
						latch_data16_5_i=1'b1;
					end
					4'b0110:
					begin
						latch_data16_6_i=1'b1;
					end
					4'b0111:
					begin
						latch_data16_7_i=1'b1;
					end
					4'b1000:
					begin
						latch_data16_8_i=1'b1;
					end
					4'b1001:
					begin
						latch_data16_9_i=1'b1;
					end
					4'b1010:
					begin
						latch_data16_A_i=1'b1;
					end
					4'b1011:
					begin
						latch_data16_B_i=1'b1;
					end
					4'b1100:
					begin
						latch_data16_C_i=1'b1;
					end
					4'b1101:
					begin
						latch_data16_D_i=1'b1;
					end
					4'b1110:
					begin
						latch_data16_E_i=1'b1;
					end
					4'b1111:
					begin
						latch_data16_F_i=1'b1;
					end
				endcase				  
			end

			PRE_WAIT_FOR_ACK_2:
				begin
					//data_from_ram_processed_i=1'b1;
					FSM2n=WAIT_FOR_ACK_2;
					request_ram_data_i=1'b1;
				end


			WAIT_FOR_ACK_2:  //wait for latching signal to drop.
			begin
				if(data_available_from_ram_gated==1'b0)
					begin
						FSM2n=WAIT_ONE_MORE_2;
					end
				else
					begin
						FSM2n=WAIT_FOR_ACK_2;
						//data_from_ram_processed_i=1'b1;
					end
			end

			WAIT_ONE_MORE_2:
			begin
				FSM2n=IDLE_2;
			end

			default:
			begin
				FSM2n=IDLE_2;
			end

		endcase  
	end //FSM2








endmodule






















