`timescale 1ns / 100ps

module top(	
				//general FPGA
				FPGA_CLK,
				FPGA_RESET,
				IO98, //goes to pin
				HARDWARE_TRIG_IP,
				LINE1,
				LINE2,

				//USB to FPGA
				IFCLK_I,
				USB_CLKOUT,
				FIFOADR_O, 
				PKTEND_O,
				PB,
				PD,
				FLAGA_I,
				FLAGB_I, 
				FLAGC_I, 
				SLRD_O, 
				SLWR_O,
				SLOE_O,

				//RAM
				RAM_ADDRESS, //A0-A19
				RAM_DATA, //IO0-IO15
				nCE,
				nOE,
				nWE,

				//DDC
				CONV,
				RESETn,
				DIN_CFG,
				CLK_CFG,
				nDVALID,
				DOUT1,
				DIN1,
				DIN2,
				DIN3,
				DIN4,
				DCLK,
				CLK
				);
				
				
/////////////////////////////////////////////////////////////////////////////////////


parameter FIRMWARE_VERSION = 16'd656;
//656 spin for revB quad board
//657 fixed PB0 and PB1 so they go to the right pins on the FPGA
//658 fixed FIFO ADDR so it doesn't go between EP2 and EP4, rather EP2 and EP6.
//659 trying out pktend
//660 usb_write_data_requested_w routed to IO98
//661 more working on that SLWR issue.  ruled out USB input, since I can send information into the FPGA and the opcode proc with handle it.
//662 trying to figure out why SLWR isn't moving when data is available
//663 removed all the SREG stuff
//664 trying to fit this design into a 250E instead of a 500E, and into a TQ144.
//665 changed DCLK wait around CONV to be register controlled from 0 out to 255 (8 bits) opcode "EB"
//667 changed DCLK wait to pause for 10uS.  16 = 3.3uS either side.
//668 changed DCLK wait to also pause for conv
//669 changed PAUSE_FOR_CONV to +/-16 from +/-8. (no help, just wider gap, as expected.
//670 added EXTRA2_OUT to opcode processor, we can now drive DCIO38 with opcode "EAxx"
//671 lots of modelsim work to get the 232 part working again, maybe at the cost of 40MHz operation.
//672 put DOUT back out and cleaned up more DCIO lines
//673 moved DOUT combinational logic after gate
//674 moved all combinational logic behind DCLK gate.
//675 gated DIN output at DIN
//676 gated DIN output in an attempt to fix glitching.
//677 cleaning up code, trying to fix DIN output from having little pulse after setting to 1'b1.
//678 moved TRIGGER_SREG to hex addr 24, bit 4 instead of bit 0.
//679 additional work on DCLK/CONV interaction.
//680 work on keeping DCLK from reading during CONV toggle.
//681 working on CFG_REG writing in DVALID_GENERATION block.
//682 working on SREG Write / Read
//683 brought out FSM from CONV to PE[3:1]
//684 worked on making CONV more stable so the software doesn't crash when trying to take data.
//685 increased CONV from 18 bit reg to 24 bit reg.  moved CONV_SEL[1:0] to address 0x57h
//686 added reset_conv feature
//687 changed defaults to be 4100 clks/conv, 80MHz/16 = 5MHz clk.
//688 changed defaults to be 1667 clks/conv, 80MHz/16 = 5MHz clk.
//689 changed CONV_OUT to be dictated by state machine, not by CONV_SEL[1]
//690 attempt #4 at non-cont mode.  worked in simulation so feeling good about this one.  worked ok.
//691 attempt #3 at non-cont mode.  fixed reset values for CONV_LOW_WAIT_REG & HIGH too. didn't work.
//692 attempt #2 at non-cont mode.  closer, but having issues with opcodes 31-34.
//693 first attempt at adding non-continuous conv mode.  didn't work.
//694 added averaging, worked.


parameter DB264 = 2'b01;


/////////////////////////////////////////////////////////////////////////////////////
// IO LINES
/////////////////////////////////////////////////////////////////////////////////////

//### Exclusively FPGA ###############################################
input FPGA_CLK; // P177 (GCLK 4 Line)
// synthesis attribute clock_signal of FPGA_CLK is "yes";
input FPGA_RESET; // P194
//input GCLK8; //P183

input USB_CLKOUT;

output IO98;
wire IO98;

input HARDWARE_TRIG_IP;

output LINE1, LINE2;
wire LINE1, LINE2;


//### CYPRESS-to-FPGA ###############################################
input IFCLK_I; // P186 (GCLK 11 Line)
// synthesis attribute clock_signal of IFCLK_I is "yes";

/*** Port A ***/
output SLOE_O;	// P9
wire   SLOE_O;


output [1:0] FIFOADR_O; // [1:0] = [ P4:P5 ]
wire   [1:0] FIFOADR_O;   

output PKTEND_O; // P3
wire   PKTEND_O;


/*** Port B ***/
inout [7:0] PB; // [7:0] = [ P40:P41:P42:P45:P47:P48:P49:P50 ]
tri   [7:0] PB;

/*** Port D ***/
inout [7:0] PD; // [7:0] = [ P15:P16:P18:P19:P22:P23:P24:P25 ]
tri   [7:0] PD;

/*** CTL Port (always input) ***/
input FLAGA_I; // P6
input FLAGB_I; // P14
input FLAGC_I;	// P26

/*** RDY Port ***/
output SLRD_O; 
wire   SLRD_O;

output SLWR_O;
wire   SLWR_O;




//### FPGA-to-MEMORY ###############################################
output [19:0] RAM_ADDRESS;
wire [19:0] RAM_ADDRESS;

inout [15:0] RAM_DATA;
tri [15:0] RAM_DATA;

output nCE, nOE, nWE;
wire nCE, nOE, nWE;



wire [1:0] board_w;
		
//### FPGA-to-DDC/DAUGHTER-CARD-HEADER #############################


output	CONV;  // 264 - CONV 
output	RESETn; // 264 - RESETn  
output	DIN_CFG; // 264 - DIN_CFG  
output	CLK_CFG; // 264 - CLK_CFG  
input		nDVALID; // 264 - DVALIDn_FROM_DDC (DVALIDn) 
input		DOUT1; // 264 - DOUT
input		DIN1;
input		DIN2;
input		DIN3;
output	DIN4; // 264 - DIN  
output	DCLK; // 264 - DCLK 
output	CLK; // 264 - CLK 

wire				CONV;
wire				RESETn;
wire				DIN_CFG;
wire				CLK_CFG;
wire				DIN4;
wire				DCLK;
wire				CLK;

//### FPGA-to-EXTRA-INPUTS ###############################################
/////////////////////////////////////////////////////////////////////////////////////

wire dclk_select_manual_or_auto_w;	//Phase3
wire dclk_manual_set_value_w;//Phase3


/////////////////////////////////////////////////////////////////////////////////////
// INTERNAL REGISTERS, WIRES, and BUSES
/////////////////////////////////////////////////////////////////////////////////////

wire [7:0] pb_i;
wire [7:0] pb_o; 
wire [7:0] pd_i;
wire [7:0] pd_o;

wire usb_io_to_opcode_processor_1_w; //goes to 1 (from USB_IO) when a word is read from the USB and available on the port.
wire opcode_processor_to_usb_io_2_w; //goes to 1	(from FPGA_FIFO) when word on port is processed.

//wire ddc_data_fifo_to_usb_io_1_w; //goes to 1 (from DDC_DATA_FIFO) when a data stream is available.
//wire usb_io_to_ddc_data_fifo_2_w; //goes to 1 (from USB_IO) when the cypress fifo is ready to accept data to write.
//wire ddc_data_fifo_to_usb_io_3_w; //goes to 1 (from DDC_DATA_FIFO) when a word is loaded on the port.
//wire usb_io_to_ddc_data_fifo_4_w; //goes to 1 (from USB_IO) when word is written to Cypress EX-USB chip.

wire [15:0] usb_io_to_opcode_processor_3_busw;
//wire [15:0] ddc_data_fifo_to_usb_io_5_busw;

wire [19:0] data_from_ddc_w;
wire [15:0] data_from_fifo_to_ram_w;
wire [15:0] data_from_fifo_to_usb_w;
wire reserve_ram_for_writing_w;
wire ddc_fifo_write_data_available_w;
wire ram_to_ddc_data_fifo_w;
//wire ddc_fifo_write_data_loaded_w;
wire data_stored_in_ram_w;
wire request_ram_data_w;
wire data_available_from_ram_w;
wire [15:0] data_from_ram_w;
wire ram_fifo_write_data_available_w;
wire usb_write_data_requested_w;
wire ram_fifo_write_data_loaded_w;
wire usb_write_data_processed_w;
wire reset_address_counters_w;

wire [15:0] opcode_data_output_w;
wire opcode_data_available_for_write_w;
wire opcode_data_loaded_w;

wire [15:0] data_for_usbio_write;



wire ddc_reset_o;
wire ddc_dvalid_i;
wire ddc_conv_o;
wire ddc_dout_i;
wire ddc_clk_cfg_o;
wire ddc_din_cfg_o;
wire ddc_clk_o;

wire CLK_CFG_w;

wire [23:0] conv_low_reg_busw;
wire [23:0] conv_high_reg_busw;
wire [15:0] conv_wait_low_reg_busw;
wire [15:0] conv_wait_high_reg_busw;
wire non_cont_w;
wire reset_conv_w;
wire [3:0] divxclk_low_reg_w;
wire [3:0] divxclk_high_reg_w;
wire [2:0] ddc_clk_sel_w;

wire dclk_from_ddc_data_cap_w;
wire [3:0] divxclk_data_low_reg_w;
wire [3:0] divxclk_data_high_reg_w;
wire [2:0] ddc_data_clk_sel_w;
wire ddc_acq_dclk_w;

wire [14:0] bits_to_read_busw;
wire format_setting_from_opcode_w;

wire cap_to_fifo_latch_data_w;
wire fifo_to_cap_data_processed_w;

wire [1:0] conv_sel_busw;

wire trigger_one_reading_w;
wire done_with_one_reading_w;
wire [7:0] dvalids_to_ignore_busw;
wire [23:0] dvalids_to_capture_busw;
wire start_conversions_w;
wire hardware_trigger_en_w;

wire done_with_conversions_w;
wire dclk_en_from_data_cap_w;

wire w_pktend_inhibitor;
wire w_pktend_sent_ack;
wire w_send_pktend;
wire w_opcode_send_pktend;

wire w_ram_data_processed;  
wire w_ready_to_write;

wire w_cont_a0_b1;


wire [15:0] dclk_wait_count_w;
wire sr_empty_gated_w;
wire trigger_dvalid;
wire dvalid_gen_manual;
wire clk_cfg_dvalid;
wire din_cfg_w;
wire [7:0] clk_cfg_high_dvalid;
wire [7:0] clk_cfg_lower_dvalid;
wire dvalid_real;
wire format_din_cfg_i;
wire [3:0] reg_freq_div_din_high_i;
wire [3:0] reg_freq_div_din_low_i;
wire start_conv;
wire [7:0] ignores_data;
wire [23:0] read_data;
wire start_conv_real;
wire [7:0] ignores_data_real;
wire [23:0] read_data_real;
//wire DIN_OUT_w;
//wire UnusedPinsOutput_w;

wire dclock_manual_w;
wire DVALIDn_FROM_DDC;

wire [1:0] first_readings;
wire reset_avgram_addr;
wire [9:0] stop_addr;
wire ab_avg_sel;
wire trigger_read_avg_ram;
wire trigger_read_opcodes;

wire [15:0] avg_data_output_w;
wire avg_data_available_for_write_w, avg_data_loaded_w;
wire use_ram_chips_w;
wire w_pause_for_conv;
wire w_state_machine_active;

wire [7:0] clkdelay_around_conv_w;



/////////////////////////////////////////////////////////////////////////////////////
// ASSIGNMENTS
/////////////////////////////////////////////////////////////////////////////////////


assign IO98=usb_write_data_requested_w;

assign LINE1=1'b0;
assign LINE2=1'b0;

assign board_w[1:0]=DB264;

//if output is enabled (sloe=0), then drive PB
//output from cypress disabled
//value of PB when FPGA outputs are enabled.
//PB are the actual xilinx pins to the cypress chip
//pb_i is the internal bus that has the input

assign PB[7:0] = (SLOE_O==1'b1) ? pb_o[7:0] : 8'bzzzzzzzz;
assign PD[7:0] = (SLOE_O==1'b1) ? pd_o[7:0] : 8'bzzzzzzzz;

//output from cypress enabled			
//value of PB when Cypress outputs are enabled.
//pb_o is the internal bus that has the output
assign pb_i[7:0] = PB[7:0];
assign pd_i[7:0] = PD[7:0];
//assign pb_i[3:0] = (SLOE_O==1'b0) ? PB[3:0] : 4'bzzzz;	    


//264 board ID = 2'h01 at opcode 22 ("2201")
//232 board ID = 2'h02 at opcode 22 ("2202")

assign CONV=ddc_conv_o;
assign RESETn=ddc_reset_o;
assign DIN_CFG=((trigger_dvalid == 1'b1)? din_cfg_w:ddc_din_cfg_o);
assign CLK_CFG=((trigger_dvalid == 1'b1)? clk_cfg_dvalid:ddc_clk_cfg_o);
assign DVALIDn_FROM_DDC = nDVALID;
assign DCLK = ((dclk_select_manual_or_auto_w==1'b1)?dclk_manual_set_value_w:(dclk_from_ddc_data_cap_w||ddc_acq_dclk_w));
assign ddc_dout_i=DOUT1;
assign CLK=ddc_clk_o;
assign DIN4 = DIN1 & DIN2 & DIN3;

// synthesis attribute clock_signal of CLK is "yes";
// synthesis attribute clock_signal of ddc_clk_o is "yes";

assign dvalid_real = (trigger_dvalid == 1'b1)? dvalid_gen_manual:ddc_dvalid_i;
assign start_conv_real = (trigger_dvalid == 1'b1)? start_conv: start_conversions_w;
assign ignores_data_real[7:0] = (trigger_dvalid == 1'b1)?ignores_data[7:0]:dvalids_to_ignore_busw[7:0];
assign read_data_real[23:0] = (trigger_dvalid == 1'b1)?read_data[23:0]:dvalids_to_capture_busw[23:0];



assign data_for_usbio_write[15:0] = (trigger_read_opcodes==1'b1 ? opcode_data_output_w[15:0] :(trigger_read_avg_ram==1'b1 ? avg_data_output_w[15:0]	: data_from_fifo_to_usb_w[15:0]));

/////////////////////////////////////////////////////////////////////////////////////
// MODULE INSTANTIATION
/////////////////////////////////////////////////////////////////////////////////////

INPUT_SYNC dvalid_sync(	.FPGA_CLK(FPGA_CLK),
								.FPGA_RESET(FPGA_RESET),
								.A(DVALIDn_FROM_DDC),
								/*.RESET_STATE(1'b0),*/
								.Y(ddc_dvalid_i),
								.CONV(ddc_conv_o),
								.CONT_A0_B1(w_cont_a0_b1)
								);


CLK_GEN ddc_clk_generator(	.FPGA_CLK(FPGA_CLK), 
									.FPGA_RESET(FPGA_RESET), 
									.DDC_CLK_OUT(ddc_clk_o), 
									.DDC_CLK_SEL(ddc_clk_sel_w), 
									.DIVXCLK_LOW_REG(divxclk_low_reg_w), 
									.DIVXCLK_HIGH_REG(divxclk_high_reg_w)
									);

//CLK_GEN ddc_data_clk_gen(	.FPGA_CLK(FPGA_CLK), 
//									.FPGA_RESET(FPGA_RESET), 
//									.DDC_CLK_OUT(dclk_from_ddc_data_cap_w), 
//									.DDC_CLK_SEL(ddc_data_clk_sel_w), 
//									.DIVXCLK_LOW_REG(divxclk_data_low_reg_w), 
//									.DIVXCLK_HIGH_REG(divxclk_data_high_reg_w)
//									);

CONV_GEN ddc_conv_generator(	.DDC_CLK(ddc_clk_o), 
										.FPGA_RESET(FPGA_RESET),
										.CONV_LOW_REG(conv_low_reg_busw[23:0]),
										.CONV_HIGH_REG(conv_high_reg_busw[23:0]),
										.CONV_WAIT_LOW_REG(conv_wait_low_reg_busw[15:0]),
										.CONV_WAIT_HIGH_REG(conv_wait_high_reg_busw[15:0]),
										.NON_CONT(non_cont_w),
										.CONV_OUT(ddc_conv_o),
										.CONV_SEL(conv_sel_busw),
										.RESET_CONV(reset_conv_w),
										.PAUSE_FOR_CONV(w_pause_for_conv),
										.CLKDELAY_AROUND_CONV(clkdelay_around_conv_w)
										);

OPCODE_PROC opcode_processor(	.FPGA_CLK(FPGA_CLK),
										.FPGA_RESET(FPGA_RESET),
										
										.USB_READ_DATA_AVAILABLE(usb_io_to_opcode_processor_1_w),
										.READ_DATA_PROCESSED(opcode_processor_to_usb_io_2_w),
										.DATA_FROM_USB(usb_io_to_opcode_processor_3_busw[15:0]),
										
										
										.CONV_LOW_REG(conv_low_reg_busw[23:0]),
										.CONV_HIGH_REG(conv_high_reg_busw[23:0]),
										.CONV_WAIT_LOW_REG(conv_wait_low_reg_busw[15:0]),
										.CONV_WAIT_HIGH_REG(conv_wait_high_reg_busw[15:0]),
										.NON_CONT(non_cont_w),
										.RESET_CONV(reset_conv_w),
										.CONV_SEL(conv_sel_busw),
										.DIVXCLK_LOW_REG(divxclk_low_reg_w),
										.DIVXCLK_HIGH_REG(divxclk_high_reg_w),
										.DDC_CLK_SEL(ddc_clk_sel_w),

										.BITS_TO_READ(bits_to_read_busw[14:0]),
										.FORMAT(format_setting_from_opcode_w),

										.DDC_DATA_CLK_SEL(ddc_data_clk_sel_w), 									
										.DIVXCLK_DATA_LOW_REG(divxclk_data_low_reg_w), 
										.DIVXCLK_DATA_HIGH_REG(divxclk_data_high_reg_w),

										.DVALIDS_TO_IGNORE(dvalids_to_ignore_busw[7:0]),
										.DVALIDS_TO_CAPTURE(dvalids_to_capture_busw[23:0]),
										.START_CONVERSIONS(start_conversions_w),
										.HARDWARE_TRIGGER_EN(hardware_trigger_en_w),

										.CLK_CFG_OUT(ddc_clk_cfg_o),
										.DIN_CFG_OUT(ddc_din_cfg_o),

										.DCLK_WAIT_COUNT(dclk_wait_count_w),
										.DDC_RESETN_OUT(ddc_reset_o),


										.TRIGGER(trigger_dvalid),
										.DIN_CFG_LOWER_BITS(clk_cfg_lower_dvalid),						 //phase 4
										.DIN_CFG_HIGHER_BITS(clk_cfg_high_dvalid),						 //phase 4

										.OPCODE_DATA_OUTPUT(opcode_data_output_w[15:0]),
										.OPCODE_DATA_AVAILABLE_FOR_WRITE(opcode_data_available_for_write_w),
										.USB_WRITE_DATA_REQUESTED(usb_write_data_requested_w),
										.OPCODE_DATA_LOADED(opcode_data_loaded_w),
										.USB_WRITE_DATA_PROCESSED(usb_write_data_processed_w),
										.DCLK_SELECT_MANUAL_OR_AUTO(dclk_select_manual_or_auto_w),
										.DCLK_MANUAL_SET_VALUE(dclk_manual_set_value_w),
										.DOUT_IN(ddc_dout_i),
										
										.FORMAT_DIN_CFG(format_din_cfg_i),
										.REG_FREQ_DIV_DIN_CLK_HIGH(reg_freq_div_din_high_i),
										.REG_FREQ_DIV_DIN_CLK_LOW(reg_freq_div_din_low_i),
										.FIRMWARE_VERSION(FIRMWARE_VERSION),
										.STOP_ADDR(stop_addr),
                              .AB_AVG_SEL(ab_avg_sel),
                              .TRIGGER_READ_AVG_RAM(trigger_read_avg_ram),
                              .READ_OUT_OPCODE_DATA_TRIGGER(trigger_read_opcodes),
										.USE_RAM_CHIPS(use_ram_chips_w),
										.DONE_READ(done_with_conversions_w),
										
										.CLKDELAY_AROUND_CONV(clkdelay_around_conv_w),
										.BOARD(board_w),
										
										.SEND_PACKET(w_opcode_send_pktend)
										);

DDC_ACQ_CONTROL ddc_acq_ctrl(	.FPGA_CLK(FPGA_CLK),
										//.DDC_CLK(ddc_clk_o),
										.FPGA_RESET(FPGA_RESET),

										.DVALIDn(dvalid_real),			  //////////////////////
										.DCLK(ddc_acq_dclk_w),

										.DVALIDS_TO_IGNORE(ignores_data_real[7:0]),	////////////////////////
										.DVALIDS_TO_CAPTURE(read_data_real[23:0]),/////////////////////////
										.START_CONVERSIONS(start_conv_real),	/////////////////////////

										.TRIGGER(trigger_one_reading_w),
										.SR_EMPTY_GATED(sr_empty_gated_w),
										.ACK(done_with_one_reading_w),
										.DONE(done_with_conversions_w),

										//.PKTEND_INHIBITOR(w_pktend_inhibitor),

										.RESERVE_RAM_FOR_WRITING(reserve_ram_for_writing_w),
										.RESET_ADDRESS_COUNTERS(reset_address_counters_w),

										.HARDWARE_TRIGGER_EN(hardware_trigger_en_w),
										.HARDWARE_TRIGGER(HARDWARE_TRIG_IP),
										
										.FIRST_READINGS(first_readings[1:0]),
										.RESET_AVGRAM_ADDR(reset_avgram_addr),
										
										.STATE_MACHINE_ACTIVE(w_state_machine_active)
										);


DDC_DATA_CAP ddc_readback(	.FPGA_CLK(FPGA_CLK),
									.FPGA_RESET(FPGA_RESET),

									.DCLK_EN(dclk_en_from_data_cap_w),
									.DOUT_IN(ddc_dout_i),

									.CAP_DATA_OUT(data_from_ddc_w[19:0]),

									.BITS_TO_READ(bits_to_read_busw[14:0]),
									.FORMAT(format_setting_from_opcode_w),

									.DATA_READY(cap_to_fifo_latch_data_w),
									.ACK(fifo_to_cap_data_processed_w),
									.READ_ONE_DATA_SET_FROM_DDC(trigger_one_reading_w),
									.DONE_READING_DATA_SET_FROM_DDC(done_with_one_reading_w),

									.DCLK_OUT(dclk_from_ddc_data_cap_w),
									.DCLK_SEL(ddc_data_clk_sel_w),
									.DIVXCLK_LOW_REG(divxclk_data_low_reg_w),
									.DIVXCLK_HIGH_REG(divxclk_data_high_reg_w),
									.DCLK_WAIT_COUNT(dclk_wait_count_w),
									
									.FIRST_READINGS(first_readings[1:0]),
									.RESET_AVGRAM_ADDR(reset_avgram_addr),

                           .STOP_ADDR(stop_addr),
                           .AB_AVG_SEL(ab_avg_sel),
                           .TRIGGER_READ_RAM(trigger_read_avg_ram),
                           
                           
							      .AVG_DATA_OUTPUT(avg_data_output_w[15:0]),
							      .AVG_DATA_AVAILABLE_FOR_WRITE(avg_data_available_for_write_w),
							      .AVG_DATA_LOADED(avg_data_loaded_w),
							      .USB_WRITE_DATA_REQUESTED(usb_write_data_requested_w),
							      .USB_WRITE_DATA_PROCESSED(usb_write_data_processed_w),
									
									.CONV_A0_B1(w_cont_a0_b1),
									.PAUSE_FOR_CONV(w_pause_for_conv && w_state_machine_active)
									);

DDC_DATA_FIFO data_fifo(.FPGA_CLK(FPGA_CLK),
								.FPGA_RESET(FPGA_RESET),

								.DATA_FROM_DDC(data_from_ddc_w[19:0]),
								.A0_OR_B1_DATA(w_cont_a0_b1),

								.DATA_WIDTH(format_setting_from_opcode_w),
								.LATCH_DATA_FROM_DDC(cap_to_fifo_latch_data_w),
								.DATA_FROM_DDC_PROCESSED(fifo_to_cap_data_processed_w),

								.DATA_FROM_FIFO_TO_RAM(data_from_fifo_to_ram_w[15:0]),

								//.RESERVE_RAM_FOR_WRITING(reserve_ram_for_writing_w),
								.SR_EMPTY_GATED(sr_empty_gated_w),
								.RAM_WRITE_DATA_PROCESSED(w_ram_data_processed),
								.READY_TO_WRITE(w_ready_to_write),

								.DDC_FIFO_WRITE_DATA_AVAILABLE(ddc_fifo_write_data_available_w),
								.RAM_WRITE_DATA_REQUESTED(ram_to_ddc_data_fifo_w)
								//.DDC_FIFO_WRITE_DATA_LOADED(ddc_fifo_write_data_loaded_w)
								);



RAM_CONTROL memory_con(	.FPGA_CLK(FPGA_CLK),
								.FPGA_RESET(FPGA_RESET),

								//
								.DATA_FROM_CAPTURE_FIFO(data_from_fifo_to_ram_w[15:0]),
								.DATA_FROM_RAM(data_from_ram_w[15:0]),
								

								.RAM_ADDR_OUT(RAM_ADDRESS[19:0]),
								//.RAM_DATA(ram_data_internal_bus_w[15:0]),
								.RAM_DATA(RAM_DATA[15:0]),
								//.PUT_DATA_ON_PORT(put_ram_data_on_port_w),

								.nCS1(nCE),
								.nOE_PASSTHROUGH(nOE),
								.nWE_PASSTHROUGH(nWE),

								// Communications with DDC_ACQ_CONTROL
								.RESET_ADDRESS_COUNTERS(reset_address_counters_w),

								// Communications with DDC_DATA_FIFO
								.RESERVE_RAM_FOR_WRITING(reserve_ram_for_writing_w),

								.DATA_FROM_DDC_FIFO_AVAILABLE(ddc_fifo_write_data_available_w),	// this is 1'b1 when the DDC_DATA_FIFO shows !empty
								.RAM_WRITE_DATA_REQUESTED(ram_to_ddc_data_fifo_w),  // this is how we say we want the data to be loaded onto DATA_FROM_CAPTURE_FIFO[15:0]
								//.DDC_FIFO_WRITE_DATA_LOADED(ddc_fifo_write_data_loaded_w), // this goes to 1 when data is loaded onto DATA_FROM_CAPTURE_FIFO[15:0]
								.RAM_WRITE_DATA_PROCESSED(w_ram_data_processed),
								.READY_TO_WRITE(w_ready_to_write),

								// Communications with RAM_TO_USB_FIFO
								.DATA_STORED_IN_RAM(data_stored_in_ram_w),
							   .REQUEST_RAM_DATA(request_ram_data_w),
								.DATA_AVAILABLE_FROM_RAM(data_available_from_ram_w),
								.USE_RAM_CHIPS(use_ram_chips_w)
								);


RAM_TO_USB_FIFO ram_fifo(	.FPGA_CLK(FPGA_CLK),
									.FPGA_RESET(FPGA_RESET),

									.DATA_FROM_RAM(data_from_ram_w[15:0]),
								
									.RAM_RESERVED_FOR_WRITING(reserve_ram_for_writing_w), //this is set when the device is running, storing data to the RAM

									.DATA_STORED_IN_RAM(data_stored_in_ram_w), //this is high when ram address in != ram address out
									.REQUEST_RAM_DATA(request_ram_data_w), // The FIFO sets this when sr_full=0 and when data_stored_in_RAM=1
									.DATA_AVAILABLE_FROM_RAM(data_available_from_ram_w), //DDC_DATA_CAPTURE sets this to 1 when new data is available for latching
									//.DATA_FROM_RAM_PROCESSED(data_from_ram_processed_w),

									.DATA_FROM_FIFO_TO_USB(data_from_fifo_to_usb_w),

									.RAM_FIFO_WRITE_DATA_AVAILABLE(ram_fifo_write_data_available_w),
									.USB_WRITE_DATA_REQUESTED(usb_write_data_requested_w),
									.RAM_FIFO_WRITE_DATA_LOADED(ram_fifo_write_data_loaded_w),
									.USB_WRITE_DATA_PROCESSED(usb_write_data_processed_w),
									
									.PKTEND_INHIBITOR(w_pktend_inhibitor),
								   .PKTEND_SEND(w_send_pktend),
   								.PKTEND_SENT(w_pktend_sent_ack)

									//.latch_out(PE[0]),
									//.fifo_address_out(PE[4:1])
									);






USB_IO usb_interface(	.IF_CLK(IFCLK_I),
								.FPGA_RESET(FPGA_RESET),

								.SLAVE_READ(SLRD_O),
								.SLAVE_WRITE(SLWR_O),
								.SLAVE_OUTPUT_ENABLE(SLOE_O),
								.FIFO_EMPTY_FLAG_C(FLAGC_I),
								.FIFO_FULL_FLAG_B(FLAGB_I),
								.FIFO_ADDRESS(FIFOADR_O),
								.PACKET_END(PKTEND_O),    

								.DATA_FROM_USB_PORT( {pb_i[7:0],pd_i[7:0]} ),
								.DATA_FROM_USB_TO_OPCODE_PROCESSOR(usb_io_to_opcode_processor_3_busw[15:0]),
								.DATA_FROM_DDC(data_for_usbio_write[15:0]),	 //see assign statement
								.DATA_TO_USB_PORT( {pb_o[7:0],pd_o[7:0]} ),

								.USB_READ_DATA_AVAILABLE(usb_io_to_opcode_processor_1_w),
								.READ_DATA_PROCESSED(opcode_processor_to_usb_io_2_w),

								.DDC_DATA_AVAILABLE_FOR_WRITE( ram_fifo_write_data_available_w || opcode_data_available_for_write_w || avg_data_available_for_write_w),
		   		 			.USB_WRITE_DATA_REQUESTED_FROM_DDC_FIFO(usb_write_data_requested_w),
								.DDC_DATA_LOADED_FOR_WRITE(ram_fifo_write_data_loaded_w || opcode_data_loaded_w || avg_data_loaded_w),
								.USB_WRITE_DATA_PROCESSED(usb_write_data_processed_w),
								.PKTEND_INHIBITOR(w_pktend_inhibitor),
								.PKTEND_SENT_ACK(w_pktend_sent_ack),
                		   .SEND_PKTEND(w_send_pktend||w_opcode_send_pktend)

								/*
								OPCODE_DATA_AVAILABLE_FOR_WRITE(),
					       	USB_WRITE_OPCODE_DATA_REQUESTED_FROM_DDC_FIFO(),
		       				OPCODE_DATA_LOADED_FOR_WRITE(),
		    					USB_WRITE_OPCODE_DATA_PROCESSED()
								*/
								);






/*CFG_DDC232 cfg_pat_gen(	.FPGA_CLK(FPGA_CLK),
								.FPGA_RESET(FPGA_RESET),
								.DIN_CFG_DATA(din_cfg_data_w[11:0]), 
								.APPLY_PATTERN(cfg_pattern_apply_w), 
								.CLK_CFG(ddc_clk_cfg_o), 
								.DIN_CFG(ddc_din_cfg_o)); */

//UNUSED_PINS unused_pin_handler(	.a3(UnusedPinsOutput_w),
//											.aIP(EXTRA_IP[7:1]),
//											.aIOSMA(EXTRA_IO_SMA),
//											.CTL(CTL),
//											.FLAGA_I(FLAGA_I)
//									

DVALID_GENERATION dvalid_generation	(.FPGA_CLK(FPGA_CLK),
												 .FPGA_RESET(FPGA_RESET),
						  						 .TRIGGER(trigger_dvalid), 
												 .CLK_CFG(clk_cfg_dvalid), 
												 .DIN_CFG(din_cfg_w),
												 .CLK_CFG_HIGH(reg_freq_div_din_high_i),
												 .CLK_CFG_LOW(reg_freq_div_din_low_i),
												 .FORMAT(format_din_cfg_i),
												 .DIN_CFG_HIGHER_BITS(clk_cfg_high_dvalid),
								 				 .DIN_CFG_LOWER_BITS(clk_cfg_lower_dvalid),
								 				 .DVALID_GEN(dvalid_gen_manual),
												 .START_C(start_conv),
								 				 .IGNORES(ignores_data),
								 				 .READING(read_data),
												 .DONE_READ(done_with_conversions_w)
												 );


/////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////
// FPGA REGISTERS THAT STORE USB OUTPUT DATA FOR USB INPUT
/////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////////////////////////////
// STATE MACHINES
/////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////






/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////


endmodule

